#include "TAlBigRIPSExample.hh"

#include <iostream>

#include "TArtAnaFile.hh"
#include "TArtAnaClock.hh"

#include "TArtCalibPID.hh"
#include "TArtCalibPPAC.hh"
#include "TArtCalibPlastic.hh"
#include "TArtCalibIC.hh"
#include "TArtCalibFocalPlane.hh"
#include "TArtPPAC.hh"
#include "TArtPlastic.hh"
#include "TArtIC.hh"
#include "TArtFocalPlane.hh"
#include "TArtBigRIPSParameters.hh"

#include "EAnalyser.hh"
#include "EWNum.hh"

#include "TVectorD.h"

TAlBigRIPSExample::TAlBigRIPSExample()
  : TArtAnaLoop(), calibpid(0), pidparameters(0)
{;}

TAlBigRIPSExample::~TAlBigRIPSExample()
{
  Destruct();
}

void TAlBigRIPSExample::Construct()
{
  //  pidparameters = new TArtBigRIPSParameters();
  pidparameters = TArtBigRIPSParameters::Instance();
  pidparameters->LoadParameter((char*)"db/BigRIPSPPAC.xml");
  pidparameters->LoadParameter((char*)"db/BigRIPSPlastic.xml");
  pidparameters->LoadParameter((char*)"db/BigRIPSIC.xml");
  pidparameters->LoadParameter((char*)"db/FocalPlane.xml");

  calibpid= new TArtCalibPID(); //must be called after parameter is constructed

  if(!this->fAnaFile){
    otree = new TTree("tree","bigrips tree");

    /* prepare tree */
    otree->Branch("F3PPAC1A_X",&F3PPAC1A_X,"F3PPAC1A_X/D");
    otree->Branch("F3PPAC1A_Y",&F3PPAC1A_Y,"F3PPAC1A_Y/D");
    otree->Branch("F3PPAC1B_X",&F3PPAC1B_X,"F3PPAC1B_X/D");
    otree->Branch("F3PPAC1B_Y",&F3PPAC1A_Y,"F3PPAC1B_Y/D");
    otree->Branch("F3PPAC2A_X",&F3PPAC2A_X,"F3PPAC2A_X/D");
    otree->Branch("F3PPAC2A_Y",&F3PPAC2A_Y,"F3PPAC2A_Y/D");
    otree->Branch("F3PPAC2B_X",&F3PPAC2B_X,"F3PPAC2B_X/D");
    otree->Branch("F3PPAC2B_Y",&F3PPAC2A_Y,"F3PPAC2B_Y/D");

    otree->Branch("F3X",&F3X,"F3X/D");
    otree->Branch("F3A",&F3A,"F3A/D");
    otree->Branch("F3Y",&F3Y,"F3Y/D");
    otree->Branch("F3B",&F3B,"F3B/D");
    
    otree->Branch("F3PLA_TL",&F3PLA_TL,"F3PLA_TL/D");
    otree->Branch("F3PLA_TR",&F3PLA_TR,"F3PLA_TR/D");
    otree->Branch("F3PLA_QL",&F3PLA_QL,"F3PLA_QL/I");
    otree->Branch("F3PLA_QR",&F3PLA_QR,"F3PLA_QR/I");

    otree->Branch("F5PPAC1A_X",&F5PPAC1A_X,"F5PPAC1A_X/D");
    otree->Branch("F5PPAC1A_Y",&F5PPAC1A_Y,"F5PPAC1A_Y/D");
    otree->Branch("F5PPAC1B_X",&F5PPAC1B_X,"F5PPAC1B_X/D");
    otree->Branch("F5PPAC1B_Y",&F5PPAC1A_Y,"F5PPAC1B_Y/D");
    otree->Branch("F5PPAC2A_X",&F5PPAC2A_X,"F5PPAC2A_X/D");
    otree->Branch("F5PPAC2A_Y",&F5PPAC2A_Y,"F5PPAC2A_Y/D");
    otree->Branch("F5PPAC2B_X",&F5PPAC2B_X,"F5PPAC2B_X/D");
    otree->Branch("F5PPAC2B_Y",&F5PPAC2A_Y,"F5PPAC2B_Y/D");

    otree->Branch("F5X",&F5X,"F5X/D");
    otree->Branch("F5A",&F5A,"F5A/D");
    otree->Branch("F5Y",&F5Y,"F5Y/D");
    otree->Branch("F5B",&F5B,"F5B/D");

    otree->Branch("F7PPAC1A_X",&F7PPAC1A_X,"F7PPAC1A_X/D");
    otree->Branch("F7PPAC1A_Y",&F7PPAC1A_Y,"F7PPAC1A_Y/D");
    otree->Branch("F7PPAC1B_X",&F7PPAC1B_X,"F7PPAC1B_X/D");
    otree->Branch("F7PPAC1B_Y",&F7PPAC1A_Y,"F7PPAC1B_Y/D");
    otree->Branch("F7PPAC2A_X",&F7PPAC2A_X,"F7PPAC2A_X/D");
    otree->Branch("F7PPAC2A_Y",&F7PPAC2A_Y,"F7PPAC2A_Y/D");
    otree->Branch("F7PPAC2B_X",&F7PPAC2B_X,"F7PPAC2B_X/D");
    otree->Branch("F7PPAC2B_Y",&F7PPAC2A_Y,"F7PPAC2B_Y/D");

    otree->Branch("F7X",&F7X,"F7X/D");
    otree->Branch("F7A",&F7A,"F7A/D");
    otree->Branch("F7Y",&F7Y,"F7Y/D");
    otree->Branch("F7B",&F7B,"F7B/D");

    otree->Branch("F7PLA_TL",&F7PLA_TL,"F7PLA_TL/D");
    otree->Branch("F7PLA_TR",&F7PLA_TR,"F7PLA_TR/D");
    otree->Branch("F7PLA_QL",&F7PLA_QL,"F7PLA_QL/I");
    otree->Branch("F7PLA_QR",&F7PLA_QR,"F7PLA_QR/I");

    otree->Branch("F7ICQ",F7ICQ,"F7ICQ[6]/I");
    otree->Branch("F7ICE",&F7ICE,"F7ICE/D");

    otree->Branch("F8PPAC1A_X",&F8PPAC1A_X,"F8PPAC1A_X/D");
    otree->Branch("F8PPAC1A_Y",&F8PPAC1A_Y,"F8PPAC1A_Y/D");
    otree->Branch("F8PPAC1B_X",&F8PPAC1B_X,"F8PPAC1B_X/D");
    otree->Branch("F8PPAC1B_Y",&F8PPAC1A_Y,"F8PPAC1B_Y/D");
    otree->Branch("F8PPAC2A_X",&F8PPAC2A_X,"F8PPAC2A_X/D");
    otree->Branch("F8PPAC2A_Y",&F8PPAC2A_Y,"F8PPAC2A_Y/D");
    otree->Branch("F8PPAC2B_X",&F8PPAC2B_X,"F8PPAC2B_X/D");
    otree->Branch("F8PPAC2B_Y",&F8PPAC2A_Y,"F8PPAC2B_Y/D");

    otree->Branch("F8X",&F8X,"F8X/D");
    otree->Branch("F8A",&F8A,"F8A/D");
    otree->Branch("F8Y",&F8Y,"F8Y/D");
    otree->Branch("F8B",&F8B,"F8B/D");

    otree->Branch("F9PPAC1A_X",&F9PPAC1A_X,"F9PPAC1A_X/D");
    otree->Branch("F9PPAC1A_Y",&F9PPAC1A_Y,"F9PPAC1A_Y/D");
    otree->Branch("F9PPAC1B_X",&F9PPAC1B_X,"F9PPAC1B_X/D");
    otree->Branch("F9PPAC1B_Y",&F9PPAC1A_Y,"F9PPAC1B_Y/D");
    otree->Branch("F9PPAC2A_X",&F9PPAC2A_X,"F9PPAC2A_X/D");
    otree->Branch("F9PPAC2A_Y",&F9PPAC2A_Y,"F9PPAC2A_Y/D");
    otree->Branch("F9PPAC2B_X",&F9PPAC2B_X,"F9PPAC2B_X/D");
    otree->Branch("F9PPAC2B_Y",&F9PPAC2A_Y,"F9PPAC2B_Y/D");

    otree->Branch("F9X",&F9X,"F9X/D");
    otree->Branch("F9A",&F9A,"F9A/D");
    otree->Branch("F9Y",&F9Y,"F9Y/D");
    otree->Branch("F9B",&F9B,"F9B/D");

    otree->Branch("F11PPAC1A_X",&F11PPAC1A_X,"F11PPAC1A_X/D");
    otree->Branch("F11PPAC1A_Y",&F11PPAC1A_Y,"F11PPAC1A_Y/D");
    otree->Branch("F11PPAC1B_X",&F11PPAC1B_X,"F11PPAC1B_X/D");
    otree->Branch("F11PPAC1B_Y",&F11PPAC1A_Y,"F11PPAC1B_Y/D");
    otree->Branch("F11PPAC2A_X",&F11PPAC2A_X,"F11PPAC2A_X/D");
    otree->Branch("F11PPAC2A_Y",&F11PPAC2A_Y,"F11PPAC2A_Y/D");
    otree->Branch("F11PPAC2B_X",&F11PPAC2B_X,"F11PPAC2B_X/D");
    otree->Branch("F11PPAC2B_Y",&F11PPAC2A_Y,"F11PPAC2B_Y/D");

    otree->Branch("F11X",&F11X,"F11X/D");
    otree->Branch("F11A",&F11A,"F11A/D");
    otree->Branch("F11Y",&F11Y,"F11Y/D");
    otree->Branch("F11B",&F11B,"F11B/D");

    otree->Branch("F11ICQ",F11ICQ,"F11ICQ[8]/I");
    otree->Branch("F11ICE",&F11ICE,"F11ICE/D");
  }

}

void TAlBigRIPSExample::Calculate()
{
  calibpid->ClearData();
  calibpid->ReconstructData();
  if(!this->fAnaFile)
    Clear();
  Fill();
}

void TAlBigRIPSExample::Destruct()
{
  delete calibpid;
  calibpid = 0;
  //  delete pidparameters;
  pidparameters->Delete();
  pidparameters = 0;
}

void TAlBigRIPSExample::Fill()
{

  TArtCalibPPAC *cppac = calibpid->GetCalibPPAC();
  TArtCalibPlastic *cpla = calibpid->GetCalibPlastic();
  TArtCalibIC *cic = calibpid->GetCalibIC();
  TArtCalibFocalPlane *cfpl = calibpid->GetCalibFocalPlane();

  using namespace Analyser;

  TArtPPAC *ppac;
  // F3-PPAC
  ppac = cppac->FindPPAC((char*)"F3PPAC-1A");
  if(ppac){
    if(this->fAnaFile){
      Add(BIGRIPS, WNum::BIGRIPS::F3PPAC1A_X, 0, ppac->GetX());
      Add(BIGRIPS, WNum::BIGRIPS::F3PPAC1A_Y, 0, ppac->GetY());
    }
    else{
      F3PPAC1A_X = ppac->GetX(); F3PPAC1A_Y = ppac->GetY();
    }
  }
  ppac = cppac->FindPPAC((char*)"F3PPAC-1B");
  if(ppac){
    if(this->fAnaFile){
      Add(BIGRIPS, WNum::BIGRIPS::F3PPAC1B_X, 0, ppac->GetX());
      Add(BIGRIPS, WNum::BIGRIPS::F3PPAC1B_Y, 0, ppac->GetY());
    }
    else{
      F3PPAC1B_X = ppac->GetX(); F3PPAC1B_Y = ppac->GetY();
    }
  }
  ppac = cppac->FindPPAC((char*)"F3PPAC-2A");
  if(ppac){
    if(this->fAnaFile){
      Add(BIGRIPS, WNum::BIGRIPS::F3PPAC2A_X, 0, ppac->GetX());
      Add(BIGRIPS, WNum::BIGRIPS::F3PPAC2A_Y, 0, ppac->GetY());
    }
    else{
      F3PPAC2A_X = ppac->GetX(); F3PPAC2A_Y = ppac->GetY();
    }
  }
  ppac = cppac->FindPPAC((char*)"F3PPAC-2B");
  if(ppac){
    if(this->fAnaFile){
      Add(BIGRIPS, WNum::BIGRIPS::F3PPAC2B_X, 0, ppac->GetX());
      Add(BIGRIPS, WNum::BIGRIPS::F3PPAC2B_Y, 0, ppac->GetY());
    }
    else{
      F3PPAC2B_X = ppac->GetX(); F3PPAC2B_Y = ppac->GetY();
    }
  }

  // F5-PPAC
  ppac = cppac->FindPPAC((char*)"F5PPAC-1A");
  if(ppac){
    if(this->fAnaFile){
      Add(BIGRIPS, WNum::BIGRIPS::F5PPAC1A_X, 0, ppac->GetX());
      Add(BIGRIPS, WNum::BIGRIPS::F5PPAC1A_Y, 0, ppac->GetY());
    }
    else{
      F5PPAC1A_X = ppac->GetX(); F5PPAC1A_Y = ppac->GetY();
    }
  }
  ppac = cppac->FindPPAC((char*)"F5PPAC-1B");
  if(ppac){
    if(this->fAnaFile){
      Add(BIGRIPS, WNum::BIGRIPS::F5PPAC1B_X, 0, ppac->GetX());
      Add(BIGRIPS, WNum::BIGRIPS::F5PPAC1B_Y, 0, ppac->GetY());
    }
    else{
      F5PPAC1B_X = ppac->GetX(); F5PPAC1B_Y = ppac->GetY();
    }
  }
  ppac = cppac->FindPPAC((char*)"F5PPAC-2A");
  if(ppac){
    if(this->fAnaFile){
      Add(BIGRIPS, WNum::BIGRIPS::F5PPAC2A_X, 0, ppac->GetX());
      Add(BIGRIPS, WNum::BIGRIPS::F5PPAC2A_Y, 0, ppac->GetY());
    }
    else{
      F5PPAC2A_X = ppac->GetX(); F5PPAC2A_Y = ppac->GetY();
    }
  }
  ppac = cppac->FindPPAC((char*)"F5PPAC-2B");
  if(ppac){
    if(this->fAnaFile){
      Add(BIGRIPS, WNum::BIGRIPS::F5PPAC2B_X, 0, ppac->GetX());
      Add(BIGRIPS, WNum::BIGRIPS::F5PPAC2B_Y, 0, ppac->GetY());
    }
    else{
      F5PPAC2B_X = ppac->GetX(); F5PPAC2B_Y = ppac->GetY();
    }
  }

  // F7-PPAC
  ppac = cppac->FindPPAC((char*)"F7PPAC-1A");
  if(ppac){
    if(this->fAnaFile){
      Add(BIGRIPS, WNum::BIGRIPS::F7PPAC1A_X, 0, ppac->GetX());
      Add(BIGRIPS, WNum::BIGRIPS::F7PPAC1A_Y, 0, ppac->GetY());
    }
    else{
      F7PPAC1A_X = ppac->GetX(); F7PPAC1A_Y = ppac->GetY();
    }
  }
  ppac = cppac->FindPPAC((char*)"F7PPAC-1B");
  if(ppac){
    if(this->fAnaFile){
      Add(BIGRIPS, WNum::BIGRIPS::F7PPAC1B_X, 0, ppac->GetX());
      Add(BIGRIPS, WNum::BIGRIPS::F7PPAC1B_Y, 0, ppac->GetY());
    }
    else{
      F7PPAC1B_X = ppac->GetX(); F7PPAC1B_Y = ppac->GetY();
    }
  }
  ppac = cppac->FindPPAC((char*)"F7PPAC-2A");
  if(ppac){
    if(this->fAnaFile){
      Add(BIGRIPS, WNum::BIGRIPS::F7PPAC2A_X, 0, ppac->GetX());
      Add(BIGRIPS, WNum::BIGRIPS::F7PPAC2A_Y, 0, ppac->GetY());
    }
    else{
      F7PPAC2A_X = ppac->GetX(); F7PPAC2A_Y = ppac->GetY();
    }
  }
  ppac = cppac->FindPPAC((char*)"F7PPAC-2B");
  if(ppac){
    if(this->fAnaFile){
      Add(BIGRIPS, WNum::BIGRIPS::F7PPAC2B_X, 0, ppac->GetX());
      Add(BIGRIPS, WNum::BIGRIPS::F7PPAC2B_Y, 0, ppac->GetY());
    }
    else{
      F7PPAC2B_X = ppac->GetX(); F7PPAC2B_Y = ppac->GetY();
    }
  }

  // F8-PPAC
  ppac = cppac->FindPPAC((char*)"F8PPAC-1A");
  if(ppac){
    if(this->fAnaFile){
      Add(ZERODEGREE, WNum::ZERODEGREE::F8PPAC1A_X, 0, ppac->GetX());
      Add(ZERODEGREE, WNum::ZERODEGREE::F8PPAC1A_Y, 0, ppac->GetY());
    }
    else{
      F8PPAC1A_X = ppac->GetX(); F8PPAC1A_Y = ppac->GetY();
    }
  }
  ppac = cppac->FindPPAC((char*)"F8PPAC-1B");
  if(ppac){
    if(this->fAnaFile){
      Add(ZERODEGREE, WNum::ZERODEGREE::F8PPAC1B_X, 0, ppac->GetX());
      Add(ZERODEGREE, WNum::ZERODEGREE::F8PPAC1B_Y, 0, ppac->GetY());
    }
    else{
      F8PPAC1B_X = ppac->GetX(); F8PPAC1B_Y = ppac->GetY();
    }
  }
  ppac = cppac->FindPPAC((char*)"F8PPAC-2A");
  if(ppac){
    if(this->fAnaFile){
      Add(ZERODEGREE, WNum::ZERODEGREE::F8PPAC2A_X, 0, ppac->GetX());
      Add(ZERODEGREE, WNum::ZERODEGREE::F8PPAC2A_Y, 0, ppac->GetY());
    }
    else{
      F8PPAC2A_X = ppac->GetX(); F8PPAC2A_Y = ppac->GetY();
    }
  }
  ppac = cppac->FindPPAC((char*)"F8PPAC-2B");
  if(ppac){
    if(this->fAnaFile){
      Add(ZERODEGREE, WNum::ZERODEGREE::F8PPAC2B_X, 0, ppac->GetX());
      Add(ZERODEGREE, WNum::ZERODEGREE::F8PPAC2B_Y, 0, ppac->GetY());
    }
    else{
      F8PPAC2B_X = ppac->GetX(); F8PPAC2B_Y = ppac->GetY();
    }
  }

  // F9-PPAC
  ppac = cppac->FindPPAC((char*)"F9PPAC-1A");
  if(ppac){
    if(this->fAnaFile){
      Add(ZERODEGREE, WNum::ZERODEGREE::F9PPAC1A_X, 0, ppac->GetX());
      Add(ZERODEGREE, WNum::ZERODEGREE::F9PPAC1A_Y, 0, ppac->GetY());
    }
    else{
      F9PPAC1A_X = ppac->GetX(); F9PPAC1A_Y = ppac->GetY();
    }
  }
  ppac = cppac->FindPPAC((char*)"F9PPAC-1B");
  if(ppac){
    if(this->fAnaFile){
      Add(ZERODEGREE, WNum::ZERODEGREE::F9PPAC1B_X, 0, ppac->GetX());
      Add(ZERODEGREE, WNum::ZERODEGREE::F9PPAC1B_Y, 0, ppac->GetY());
    }
    else{
      F9PPAC1B_X = ppac->GetX(); F9PPAC1B_Y = ppac->GetY();
    }
  }
  ppac = cppac->FindPPAC((char*)"F9PPAC-2A");
  if(ppac){
    if(this->fAnaFile){
      Add(ZERODEGREE, WNum::ZERODEGREE::F9PPAC2A_X, 0, ppac->GetX());
      Add(ZERODEGREE, WNum::ZERODEGREE::F9PPAC2A_Y, 0, ppac->GetY());
    }
    else{
      F9PPAC2A_X = ppac->GetX(); F9PPAC2A_Y = ppac->GetY();
    }
  }
  ppac = cppac->FindPPAC((char*)"F9PPAC-2B");
  if(ppac){
    if(this->fAnaFile){
      Add(ZERODEGREE, WNum::ZERODEGREE::F9PPAC2B_X, 0, ppac->GetX());
      Add(ZERODEGREE, WNum::ZERODEGREE::F9PPAC2B_Y, 0, ppac->GetY());
    }
    else{
      F9PPAC2B_X = ppac->GetX(); F9PPAC2B_Y = ppac->GetY();
    }
  }

  // F11-PPAC
  ppac = cppac->FindPPAC((char*)"F11PPAC-1A");
  if(ppac){
    if(this->fAnaFile){
      Add(ZERODEGREE, WNum::ZERODEGREE::F11PPAC1A_X, 0, ppac->GetX());
      Add(ZERODEGREE, WNum::ZERODEGREE::F11PPAC1A_Y, 0, ppac->GetY());
    }
    else{
      F11PPAC1A_X = ppac->GetX(); F11PPAC1A_Y = ppac->GetY();
    }
  }
  ppac = cppac->FindPPAC((char*)"F11PPAC-1B");
  if(ppac){
    if(this->fAnaFile){
      Add(ZERODEGREE, WNum::ZERODEGREE::F11PPAC1B_X, 0, ppac->GetX());
      Add(ZERODEGREE, WNum::ZERODEGREE::F11PPAC1B_Y, 0, ppac->GetY());
    }
    else{
      F11PPAC1B_X = ppac->GetX(); F11PPAC1B_Y = ppac->GetY();
    }
  }
  ppac = cppac->FindPPAC((char*)"F11PPAC-2A");
  if(ppac){
    if(this->fAnaFile){
      Add(ZERODEGREE, WNum::ZERODEGREE::F11PPAC2A_X, 0, ppac->GetX());
      Add(ZERODEGREE, WNum::ZERODEGREE::F11PPAC2A_Y, 0, ppac->GetY());
    }
    else{
      F11PPAC2A_X = ppac->GetX(); F11PPAC2A_Y = ppac->GetY();
    }
  }
  ppac = cppac->FindPPAC((char*)"F11PPAC-2B");
  if(ppac){
    if(this->fAnaFile){
      Add(ZERODEGREE, WNum::ZERODEGREE::F11PPAC2B_X, 0, ppac->GetX());
      Add(ZERODEGREE, WNum::ZERODEGREE::F11PPAC2B_Y, 0, ppac->GetY());
    }
    else{
      F11PPAC2B_X = ppac->GetX(); F11PPAC2B_Y = ppac->GetY();
    }
  }

  TArtPlastic *pla;
  // F3-Plastic
  pla = cpla->FindPlastic((char*)"F3pl");
  if(pla){
    if(this->fAnaFile){
      Add(BIGRIPS, WNum::BIGRIPS::F3PLA_TL, 0, pla->GetTimeL());
      Add(BIGRIPS, WNum::BIGRIPS::F3PLA_TR, 0, pla->GetTimeR());
      Add(BIGRIPS, WNum::BIGRIPS::F3PLA_QL, 0, pla->GetQLRaw());
      Add(BIGRIPS, WNum::BIGRIPS::F3PLA_QR, 0, pla->GetQRRaw());
    }
    else {
      F3PLA_TL = pla->GetTimeL(); F3PLA_TR = pla->GetTimeR();
      F3PLA_QL = pla->GetQLRaw(); F3PLA_QR = pla->GetQRRaw();
    }
  }
  // F7-Plastic
  pla = cpla->FindPlastic((char*)"F7pl");
  if(pla){
    if(this->fAnaFile){
      Add(BIGRIPS, WNum::BIGRIPS::F7PLA_TL, 0, pla->GetTimeL());
      Add(BIGRIPS, WNum::BIGRIPS::F7PLA_TR, 0, pla->GetTimeR());
      Add(BIGRIPS, WNum::BIGRIPS::F7PLA_QL, 0, pla->GetQLRaw());
      Add(BIGRIPS, WNum::BIGRIPS::F7PLA_QR, 0, pla->GetQRRaw());
    }
    else {
      F7PLA_TL = pla->GetTimeL(); F7PLA_TR = pla->GetTimeR();
      F7PLA_QL = pla->GetQLRaw(); F7PLA_QR = pla->GetQRRaw(); 
    }
  }

  TArtIC * ic;
  // F7-IC
  ic = cic->FindIC((char*)"F7IC");
  if(ic){
  if(this->fAnaFile){
    for(int i=0; i<6; ++i)
      Add(BIGRIPS, WNum::BIGRIPS::F7ICQ, i+1, ic->GetRawADC(i));
  }
  else{
    for(int i=0; i<6; ++i) F7ICQ[i] = ic->GetRawADC(i);
    F7ICE = ic->GetEnergySqSum();
  }
  }

  // F11-IC
  ic = cic->FindIC((char*)"F7IC");
  if(ic){
    if(this->fAnaFile){
      for(int i=0; i<8; ++i)
	Add(ZERODEGREE, WNum::ZERODEGREE::F11ICQ, i+1, ic->GetRawADC(i));
    }
    else{
      for(int i=0; i<8; ++i) F11ICQ[i] = ic->GetRawADC(i);
      F11ICE = ic->GetEnergySqSum();
    }
  }

  TArtFocalPlane *fpl; TVectorD *vec;
  // F3-Pos
  fpl = cfpl->FindFocalPlane(3); 
  if(fpl){
    vec=fpl->GetOptVector(); 
    if(this->fAnaFile){
      Add(BIGRIPS, WNum::BIGRIPS::F3X, 0, (*vec)(0));
      Add(BIGRIPS, WNum::BIGRIPS::F3A, 0, (*vec)(1));
      Add(BIGRIPS, WNum::BIGRIPS::F3Y, 0, (*vec)(2));
      Add(BIGRIPS, WNum::BIGRIPS::F3B, 0, (*vec)(3));
    }
    else{
      F3X = (*vec)(0); F3A = (*vec)(1); F3Y = (*vec)(2); F3B = (*vec)(3);
    }
  }

  // F5-Pos
  fpl = cfpl->FindFocalPlane(5); 
  if(fpl){
    vec=fpl->GetOptVector(); 
    if(this->fAnaFile){
      Add(BIGRIPS, WNum::BIGRIPS::F5X, 0, (*vec)(0));
      Add(BIGRIPS, WNum::BIGRIPS::F5A, 0, (*vec)(1));
      Add(BIGRIPS, WNum::BIGRIPS::F5Y, 0, (*vec)(2));
      Add(BIGRIPS, WNum::BIGRIPS::F5B, 0, (*vec)(3));
    }
    else {
      F5X = (*vec)(0); F5A = (*vec)(1); F5Y = (*vec)(2); F5B = (*vec)(3);
    }
  }

  // F7-Pos
  fpl = cfpl->FindFocalPlane(7); 
  if(fpl){vec=fpl->GetOptVector(); 
    if(this->fAnaFile){
      Add(BIGRIPS, WNum::BIGRIPS::F7X, 0, (*vec)(0));
      Add(BIGRIPS, WNum::BIGRIPS::F7A, 0, (*vec)(1));
      Add(BIGRIPS, WNum::BIGRIPS::F7Y, 0, (*vec)(2));
      Add(BIGRIPS, WNum::BIGRIPS::F7B, 0, (*vec)(3));
    }
    else{
      F7X = (*vec)(0); F7A = (*vec)(1); F7Y = (*vec)(2); F7B = (*vec)(3);
    }
  }

  // F8-Pos
  fpl = cfpl->FindFocalPlane(7); 
  if(fpl){vec=fpl->GetOptVector(); 
    if(this->fAnaFile){
      Add(ZERODEGREE, WNum::ZERODEGREE::F8X, 0, (*vec)(0));
      Add(ZERODEGREE, WNum::ZERODEGREE::F8A, 0, (*vec)(1));
      Add(ZERODEGREE, WNum::ZERODEGREE::F8Y, 0, (*vec)(2));
      Add(ZERODEGREE, WNum::ZERODEGREE::F8B, 0, (*vec)(3));
    }
    else{
      F8X = (*vec)(0); F8A = (*vec)(1); F8Y = (*vec)(2); F8B = (*vec)(3);
    }
  }

  // F9-Pos
  fpl = cfpl->FindFocalPlane(7); 
  if(fpl){vec=fpl->GetOptVector(); 
    if(this->fAnaFile){
      Add(ZERODEGREE, WNum::ZERODEGREE::F9X, 0, (*vec)(0));
      Add(ZERODEGREE, WNum::ZERODEGREE::F9A, 0, (*vec)(1));
      Add(ZERODEGREE, WNum::ZERODEGREE::F9Y, 0, (*vec)(2));
      Add(ZERODEGREE, WNum::ZERODEGREE::F9B, 0, (*vec)(3));
    }
    else{
      F9X = (*vec)(0); F9A = (*vec)(1); F9Y = (*vec)(2); F9B = (*vec)(3);
    }
  }

  // F11-Pos
  fpl = cfpl->FindFocalPlane(7); 
  if(fpl){vec=fpl->GetOptVector(); 
    if(this->fAnaFile){
      Add(ZERODEGREE, WNum::ZERODEGREE::F11X, 0, (*vec)(0));
      Add(ZERODEGREE, WNum::ZERODEGREE::F11A, 0, (*vec)(1));
      Add(ZERODEGREE, WNum::ZERODEGREE::F11Y, 0, (*vec)(2));
      Add(ZERODEGREE, WNum::ZERODEGREE::F11B, 0, (*vec)(3));
    }
    else{
      F11X = (*vec)(0); F11A = (*vec)(1); F11Y = (*vec)(2); F11B = (*vec)(3);
    }
  }

  if(!this->fAnaFile) otree->Fill();

}

void TAlBigRIPSExample::Clear()
{
  F3PPAC1A_X=-9999; F3PPAC1A_Y=-9999; F3PPAC1B_X=-9999; F3PPAC1B_Y=-9999;
  F3PPAC2A_X=-9999; F3PPAC2A_Y=-9999; F3PPAC2B_X=-9999; F3PPAC2B_Y=-9999;
  F5PPAC1A_X=-9999; F5PPAC1A_Y=-9999; F5PPAC1B_X=-9999; F5PPAC1B_Y=-9999;
  F5PPAC2A_X=-9999; F5PPAC2A_Y=-9999; F5PPAC2B_X=-9999; F5PPAC2B_Y=-9999;
  F7PPAC1A_X=-9999; F7PPAC1A_Y=-9999; F7PPAC1B_X=-9999; F7PPAC1B_Y=-9999;
  F7PPAC2A_X=-9999; F7PPAC2A_Y=-9999; F7PPAC2B_X=-9999; F7PPAC2B_Y=-9999;
  F8PPAC1A_X=-9999; F8PPAC1A_Y=-9999; F8PPAC1B_X=-9999; F8PPAC1B_Y=-9999;
  F8PPAC2A_X=-9999; F8PPAC2A_Y=-9999; F8PPAC2B_X=-9999; F8PPAC2B_Y=-9999;
  F9PPAC1A_X=-9999; F9PPAC1A_Y=-9999; F9PPAC1B_X=-9999; F9PPAC1B_Y=-9999;
  F9PPAC2A_X=-9999; F9PPAC2A_Y=-9999; F9PPAC2B_X=-9999; F9PPAC2B_Y=-9999;
  F11PPAC1A_X=-9999; F11PPAC1A_Y=-9999; F11PPAC1B_X=-9999; F11PPAC1B_Y=-9999;
  F11PPAC2A_X=-9999; F11PPAC2A_Y=-9999; F11PPAC2B_X=-9999; F11PPAC2B_Y=-9999;
  F3PLA_TL=-9999; F3PLA_TR=-9999; F3PLA_QL=-9999; F3PLA_QR=-9999; 
  F7PLA_TL=-9999; F7PLA_TR=-9999; F7PLA_QL=-9999; F7PLA_QR=-9999; 
  for(int i=0;i<6;i++)F7ICQ[i] = -9999;
  for(int i=0;i<8;i++)F11ICQ[i] = -9999;
  F7ICE = -9999; F11ICE = -9999;
  F3X=-9999; F3A=-9999; F3Y=-9999; F3B=-9999;
  F5X=-9999; F5A=-9999; F5Y=-9999; F5B=-9999;
  F7X=-9999; F7A=-9999; F7Y=-9999; F7B=-9999;
  F8X=-9999; F8A=-9999; F8Y=-9999; F8B=-9999;
  F9X=-9999; F9A=-9999; F9Y=-9999; F9B=-9999;
  F11X=-9999; F11A=-9999; F11Y=-9999; F11B=-9999;
}

const char* TAlBigRIPSExample::ClassName() const
{
  return "TAlBigRIPSExample";
}

ClassImp(TAlBigRIPSExample)
