#include "TAlDALIExample.hh"

#include <iostream>

#include "TClonesArray.h"
#include "TMath.h"

#include "TArtAnaFile.hh"
#include "TArtAnaClock.hh"
#include "TArtAnaLoopManager.hh"

#include "TArtCalibDALI.hh"
#include "TArtDALINaI.hh"
#include "TArtDALIParameters.hh"

#include "EAnalyser.hh"
#include "EWNum.hh"

#include "TArtStoreManager.hh"

TAlDALIExample::TAlDALIExample()
  : TArtAnaLoop(), daliparameters(0), calibdali(0)
{;}

TAlDALIExample::~TAlDALIExample()
{
  Destruct();
}

void TAlDALIExample::Construct()
{
  //  daliparameters = new TArtDALIParameters;
  daliparameters = TArtDALIParameters::Instance();
  daliparameters->LoadParameter((char*)"db/DALI.xml");
  daliparameters->PrintListOfNaIPara();
  calibdali = new TArtCalibDALI; //must be called after parameter is constructed

  if(!this->fAnaFile){
    hnai_adc = new TH2F("nai_adc","nai_adc",1000,0,4000,140,0,140);
    hnai_tdc = new TH2F("nai_tdc","nai_tdc",1000,0,4000,140,0,140);
  }else{
    /* nothing */;
  }
}

void TAlDALIExample::Calculate()
{
  calibdali->ClearData();
  calibdali->ReconstructData();
  //  calibdali->DopplerCorrect(0.616);
  calibdali->DopplerCorrect(0.602);
  using namespace Analyser;
  using namespace WNum::DALI;

//  if(this->fAnaFile){
//    Add(Analyser::GLOBAL, WNum::GLOBAL::EVENTNUM, 0,
//	TArtAnaLoopStatic::GetTotalEventNumber());
//  }

  TArtStoreManager * sman = TArtStoreManager::Instance();
  TClonesArray *nais = (TClonesArray *)sman->FindDataContainer("DALINaI");

  for(int i=0;i<nais->GetEntries();i++){
    TArtDALINaI *nai = (TArtDALINaI *)nais->At(i);

    Int_t id = nai->GetID();
    if(!this->fAnaFile){
      hnai_adc->Fill(nai->GetRawADC(), id);
      hnai_tdc->Fill(nai->GetRawTDC(), id);
    }else{
      Add(DALI, ID,            id, id);
      Add(DALI, Layer,         id, nai->GetLayer());
      Add(DALI, Theta,         id, nai->GetTheta());
      Add(DALI, CosTheta,      id, nai->GetCosTheta());

      Add(DALI, RawADC,        id, nai->GetRawADC());
      Add(DALI, Energy,        id, nai->GetEnergy());
      Add(DALI, DoppCorEnergy, id, nai->GetDoppCorEnergy());

      Add(DALI, RawTDC,        id, nai->GetRawTDC());
      Add(DALI, Time,          id, nai->GetTime());
      Add(DALI, TimeOffseted,  id, nai->GetTimeOffseted());

      Add(DALI, TimeTrueEnergy,        id, nai->GetTimeTrueEnergy());
      Add(DALI, TimeTrueDoppCorEnergy, id, nai->GetTimeTrueDoppCorEnergy());
      Add(DALI, TimeTrueTime,          id, nai->GetTimeTrueTime());
      Add(DALI, TimeTrueTimeOffseted,  id, nai->GetTimeTrueTimeOffseted());


    }      
  }
}

void TAlDALIExample::Destruct()
{
  delete calibdali;
  calibdali = 0;
  //  delete daliparameters;
  daliparameters->Delete();
  daliparameters = 0;
}

const char* TAlDALIExample::ClassName() const
{
  return "TAlDALIExample";
}
