
#include "TAlNEBULAExample.hh"

#include <iostream>

#include "TArtCore.hh"
#include "TArtAnaFile.hh"
#include "TArtAnaClock.hh"
#include "TArtAnaLoopManager.hh"

//#include "TArtCalibCoin.hh"
#include "TArtCalibNEBULA.hh"
#include "TArtEventInfo.hh"
#include "TArtNEBULAPla.hh"
#include "TArtNEBULAPlaPara.hh"
#include "TArtNEBULAFilter.hh"
#include "TArtSAMURAIParameters.hh"

#include "EAnalyser.hh"
#include "EWNum.hh"

#include "TArtStoreManager.hh"

TAlNEBULAExample::TAlNEBULAExample()
  : TArtAnaLoop(), calibnebula(0), fFilterFlag(0x0003)
{;}

TAlNEBULAExample::TAlNEBULAExample(int fFilterFlag)
  : TArtAnaLoop(), calibnebula(0), samuraiparameters(0), fFilterFlag(fFilterFlag)
{;}

TAlNEBULAExample::~TAlNEBULAExample()
{
  Destruct();
}

void TAlNEBULAExample::Construct()
{
  //  samuraiparameters = new TArtSAMURAIParameters;
  samuraiparameters = TArtSAMURAIParameters::Instance();
  //  samuraiparameters->LoadParameter("db/NEBULA_toffzero.xml");
  //  samuraiparameters->LoadParameter((char*)"db/NEBULA_ryuki.xml");
  samuraiparameters->LoadParameter((char*)"db/NEBULA.xml");
  //  calibcoin = new TArtCalibCoin; 
  calibnebula = new TArtCalibNEBULA; //must be called after parameter is constructed

  std::string nebulafiltermessage= "\n";
  if(fFilterFlag & 0x0001){
    nebulafiltermessage += " TArtNEBULAFilter::IHitMin: on\n";
  }
  if(fFilterFlag & 0x0002){
    nebulafiltermessage += " TArtNEBULAFilter::Threshold: on\n";
  }
  if(fFilterFlag & 0x0004){
    nebulafiltermessage += " TArtNEBULAFilter::Veto: on\n";
  }
  if(fFilterFlag & 0x0008){  
    nebulafiltermessage += " TArtNEBULAFilter::HitMinPos: on\n";
  }
  if(fFilterFlag & 0x0010){  
    nebulafiltermessage += " TArtNEBULAFilter::HitMinPos2: on\n";
  }
  if(nebulafiltermessage.size() > 1){
    std::cout << nebulafiltermessage << std::endl;
  }
}

void TAlNEBULAExample::Calculate()
{
  calibnebula->ClearData();

  //  calibcoin->LoadData();
  calibnebula->LoadRawData();
  calibnebula->ReconstructData();

  if(fFilterFlag & 0x0001){
    TArtNEBULAFilter::IHitMin(calibnebula, 4, 1); // neut>=4hit, veto>=1hit
  }
  if(fFilterFlag & 0x0002){
    TArtNEBULAFilter::Threshold(calibnebula, 5, -1);
  }
  if(fFilterFlag & 0x0004){  
    TArtNEBULAFilter::Veto(calibnebula, 2); // two layer
  }
  if(fFilterFlag & 0x0008){  
    TArtNEBULAFilter::HitMinPos(calibnebula); // get front hit 
  }
  if(fFilterFlag & 0x0010){  
    TArtNEBULAFilter::HitMinPos2(calibnebula); // get front hit in each layer
  }

  calibnebula->CalMultiplicity();  
  calibnebula->CalNeutron(); // only 1n, befor call this event have to be selected to one.

  //  FillCoin();
  FillNEBULA();
}

void TAlNEBULAExample::Destruct()
{
  delete calibnebula;
  calibnebula = 0;
//  delete calibcoin;
//  calibcoin = 0;
  //  delete samuraiparameters;
  samuraiparameters->Delete();
  samuraiparameters = 0;
}

// void TAlNEBULAExample::FillCoin()
// {
//   TArtStoreManager * sman = TArtStoreManager::Instance();
//   TClonesArray *info = (TClonesArray*)sman->FindDataContainer("EventInfo");
//   int runnumber = ((TArtEventInfo *)info->At(0))->GetRunNumber();
//   int evenumber = ((TArtEventInfo *)info->At(0))->GetEventNumber();
//   unsigned int tbit = ((TArtEventInfo *)info->At(0))->GetTriggerBit();
//   //  TArtCore::Info(__FILE__,"tbit: %x", tbit);
// 
//   using namespace Analyser;
//   using namespace WNum::GLOBAL;  
// 
//   Add(GLOBAL, RUNNUM,           0, (Double_t)runnumber);
//   Add(GLOBAL, EVENTNUM,         0, (Double_t)evenumber);
//   Add(GLOBAL, TRIGGERBIT,       0, (Double_t)tbit);
// 
// }

void TAlNEBULAExample::FillNEBULA()
{
  using namespace Analyser;
  using namespace WNum::NEBULA;  

  Int_t NumNEBULAPla = calibnebula->GetNumNEBULAPla();
  for(int i=0; i<NumNEBULAPla; ++i){
    TArtNEBULAPla* pla = calibnebula->GetNEBULAPla(i);
    if(pla->GetDataState() < 0) continue;

    Int_t id    = pla->GetID();

    if(!this->fAnaFile){
      /*nothing*/;
    }else{
      Add(NEBULA, ID,       id, id);
      Add(NEBULA, Layer,    id, pla->GetLayer());
      Add(NEBULA, SubLayer, id, pla->GetSubLayer());
      Add(NEBULA, DetPosX,  id, pla->GetDetPos(0));
      Add(NEBULA, DetPosY,  id, pla->GetDetPos(1));
      Add(NEBULA, DetPosZ,  id, pla->GetDetPos(2));
      Add(NEBULA, Hit,      id, pla->GetHit());

      Add(NEBULA, QURaw,        id, pla->GetQURaw());
      Add(NEBULA, QDRaw,        id, pla->GetQDRaw());
      Add(NEBULA, QUPed,        id, pla->GetQUPed());
      Add(NEBULA, QDPed,        id, pla->GetQDPed());
      Add(NEBULA, QUCal,        id, pla->GetQUCal());
      Add(NEBULA, QDCal,        id, pla->GetQDCal());
      Add(NEBULA, QAvePed,      id, pla->GetQAvePed());
      Add(NEBULA, QAveCal,      id, pla->GetQAveCal());
      Add(NEBULA, LogQPed,      id, pla->GetLogQPed());
      Add(NEBULA, LogQCal,      id, pla->GetLogQCal());
      Add(NEBULA, IvSqrtQUPed,  id, pla->GetIvSqrtQUPed());
      Add(NEBULA, IvSqrtQDPed,  id, pla->GetIvSqrtQDPed());
      Add(NEBULA, IvSqrtQAvePed,id, pla->GetIvSqrtQAvePed());

      Add(NEBULA, TURaw,       id, pla->GetTURaw());
      Add(NEBULA, TDRaw,       id, pla->GetTDRaw());
      Add(NEBULA, TUCal,       id, pla->GetTUCal());
      Add(NEBULA, TDCal,       id, pla->GetTDCal());
      Add(NEBULA, TUSlw,       id, pla->GetTUSlw());
      Add(NEBULA, TDSlw,       id, pla->GetTDSlw());
      Add(NEBULA, DTRaw,       id, pla->GetDTRaw());
      Add(NEBULA, DTCal,       id, pla->GetDTCal());
      Add(NEBULA, DTSlw,       id, pla->GetDTSlw());
      Add(NEBULA, TAveRaw,     id, pla->GetTAveRaw());
      Add(NEBULA, TAveCal,     id, pla->GetTAveCal());
      Add(NEBULA, TAveSlw,     id, pla->GetTAveSlw());
      Add(NEBULA, TTOFGamma,   id, pla->GetTTOFGamma());
      Add(NEBULA, TTOFNeutron, id, pla->GetTTOFNeutron());

      Add(NEBULA, PosCal,       id, pla->GetPosCal());
      Add(NEBULA, PosSlw,       id, pla->GetPosSlw());
      Add(NEBULA, PosX,         id, pla->GetPos(0));
      Add(NEBULA, PosY,         id, pla->GetPos(1));
      Add(NEBULA, PosZ,         id, pla->GetPos(2));
      Add(NEBULA, FlightLength, id, pla->GetFlightLength());
    }
  }

  if(!this->fAnaFile){
    /*nothing*/;
  }else{
    Add(NEBULA, Multiplicity_N,   0, calibnebula->GetMultiplicity_N());
    Add(NEBULA, Multiplicity_N1,  0, calibnebula->GetMultiplicity_N1());
    Add(NEBULA, Multiplicity_N2,  0, calibnebula->GetMultiplicity_N2());
    Add(NEBULA, Multiplicity_V1,  0, calibnebula->GetMultiplicity_V1());
    Add(NEBULA, Multiplicity_V2,  0, calibnebula->GetMultiplicity_V2());
    Add(NEBULA, Multiplicity_N11, 0, calibnebula->GetMultiplicity_N11());
    Add(NEBULA, Multiplicity_N12, 0, calibnebula->GetMultiplicity_N12());
    Add(NEBULA, Multiplicity_N21, 0, calibnebula->GetMultiplicity_N21());
    Add(NEBULA, Multiplicity_N22, 0, calibnebula->GetMultiplicity_N22());

    if(NumNEBULAPla == 1){ 
      using namespace WNum::NEUTRON;

      Add(NEUTRON, Q,      0, calibnebula->GetNeutron()->fQ);
      Add(NEUTRON, T,      0, calibnebula->GetNeutron()->fT);
      Add(NEUTRON, Betax,  0, calibnebula->GetNeutron()->fBeta[0]);
      Add(NEUTRON, Betay,  0, calibnebula->GetNeutron()->fBeta[1]);
      Add(NEUTRON, Betaz,  0, calibnebula->GetNeutron()->fBeta[2]);
      Add(NEUTRON, Gammax, 0, calibnebula->GetNeutron()->fGamma[0]);
      Add(NEUTRON, Gammay, 0, calibnebula->GetNeutron()->fGamma[1]);
      Add(NEUTRON, Gammaz, 0, calibnebula->GetNeutron()->fGamma[2]);
      Add(NEUTRON, Px,     0, calibnebula->GetNeutron()->fP[0]);
      Add(NEUTRON, Py,     0, calibnebula->GetNeutron()->fP[1]);
      Add(NEUTRON, Pz,     0, calibnebula->GetNeutron()->fP[2]);
      Add(NEUTRON, P,      0, calibnebula->GetNeutron()->fPabs);
      Add(NEUTRON, E,      0, calibnebula->GetNeutron()->fE);
      Add(NEUTRON, Theta,  0, calibnebula->GetNeutron()->fTheta);
      Add(NEUTRON, Phi,    0, calibnebula->GetNeutron()->fPhi);
     }
  }
}

void TAlNEBULAExample::SetFilterFlag(int filterflag)
{
  fFilterFlag = filterflag;
}

const char* TAlNEBULAExample::ClassName() const
{
  return "TAlNEBULAExample";
}
