#include "TAlSAMURAIPlaExample.hh"

#include <iostream>

#include "TClonesArray.h"
#include "TMath.h"

#include "TArtAnaFile.hh"
#include "TArtAnaClock.hh"

#include "TArtCalibSAMURAIPla.hh"
#include "TArtPlastic.hh"
#include "TArtSAMURAIParameters.hh"

#include "EAnalyser.hh"
#include "EWNum.hh"

#include "TArtStoreManager.hh"

TAlSAMURAIPlaExample::TAlSAMURAIPlaExample() : TArtAnaLoop()
{;}

TAlSAMURAIPlaExample::~TAlSAMURAIPlaExample()
{
  Destruct();
}

void TAlSAMURAIPlaExample::Construct()
{
  //  samuraiparameters = new TArtSAMURAIParameters;
  samuraiparameters = TArtSAMURAIParameters::Instance();
  samuraiparameters->LoadParameter((char*)"db/SAMURAIPlastic.xml");
  //  samuraiparameters->PrintListOfPlasticPara();
  calibsamuraipla = new TArtCalibSAMURAIPla; //must be called after parameter is constructed

  if(!this->fAnaFile){
    hf3pla_tl = new TH1F("f3pla_tl","f3pla_tl",1000,0,4000);
    hf3pla_tr = new TH1F("f3pla_tr","f3pla_tr",1000,0,4000);
    hf3pla_ql = new TH1F("f3pla_ql","f3pla_ql",1000,0,4000);
    hf3pla_qr = new TH1F("f3pla_qr","f3pla_qr",1000,0,4000);

    hf7pla_tl = new TH1F("f7pla_tl","f7pla_tl",1000,0,4000);
    hf7pla_tr = new TH1F("f7pla_tr","f7pla_tr",1000,0,4000);
    hf7pla_ql = new TH1F("f7pla_ql","f7pla_ql",1000,0,4000);
    hf7pla_qr = new TH1F("f7pla_qr","f7pla_qr",1000,0,4000);

    hf13pla1_tl = new TH1F("f13pla1_tl","f13pla1_tl",1000,0,4000);
    hf13pla1_tr = new TH1F("f13pla1_tr","f13pla1_tr",1000,0,4000);
    hf13pla1_ql = new TH1F("f13pla1_ql","f13pla1_ql",1000,0,4000);
    hf13pla1_qr = new TH1F("f13pla1_qr","f13pla1_qr",1000,0,4000);

    hf13pla2_tl = new TH1F("f13pla2_tl","f13pla2_tl",1000,0,4000);
    hf13pla2_tr = new TH1F("f13pla2_tr","f13pla2_tr",1000,0,4000);
    hf13pla2_ql = new TH1F("f13pla2_ql","f13pla2_ql",1000,0,4000);
    hf13pla2_qr = new TH1F("f13pla2_qr","f13pla2_qr",1000,0,4000);

    hf13sbv_tl = new TH1F("f13sbv_tl","f13sbv_tl",1000,0,4000);
    hf13sbv_tr = new TH1F("f13sbv_tr","f13sbv_tr",1000,0,4000);
    hf13sbv_ql = new TH1F("f13sbv_ql","f13sbv_ql",1000,0,4000);
    hf13sbv_qr = new TH1F("f13sbv_qr","f13sbv_qr",1000,0,4000);

    ntp = new TNtuple("ntp","samurai plastic ntuple","f3pla_tl:f3pla_tr:f3pla_q:f7pla_tl:f7pla_tr:f7pla_q:f13pla1_tl:f13pla1_tr:f13pla1_q:f13pla2_tl:f13pla2_tr:f13pla2_q:f13sbv_tl:f13sbv_tr:f13sbv_q");
  }else{
    /* nothing */;
  }
}

void TAlSAMURAIPlaExample::Calculate()
{
  calibsamuraipla->ClearData();
  calibsamuraipla->ReconstructData();

  Double_t f3pla_tl = -9999;
  Double_t f3pla_tr = -9999;
  Double_t f3pla_t = -9999;
  Double_t f3pla_dt = -9999;
  Double_t f3pla_q = -9999;
  Double_t f3pla_lnq = -9999;
  Double_t f7pla_tl = -9999;
  Double_t f7pla_tr = -9999;
  Double_t f7pla_t = -9999;
  Double_t f7pla_dt = -9999;
  Double_t f7pla_q = -9999;
  Double_t f7pla_lnq = -9999;
  Double_t f13pla1_tl = -9999;
  Double_t f13pla1_tr = -9999;
  Double_t f13pla1_t  = -9999;
  Double_t f13pla1_dt = -9999;
  Double_t f13pla1_q = -9999;
  Double_t f13pla1_lnq = -9999;
  Double_t f13pla2_tl = -9999;
  Double_t f13pla2_tr = -9999;
  Double_t f13pla2_t  = -9999;
  Double_t f13pla2_dt = -9999;
  Double_t f13pla2_q = -9999;
  Double_t f13pla2_lnq = -9999;
  Double_t f13sbv_tl = -9999;
  Double_t f13sbv_tr = -9999;
  Double_t f13sbv_t  = -9999;
  Double_t f13sbv_dt = -9999;
  Double_t f13sbv_q = -9999;
  Double_t f13sbv_lnq = -9999;

  using namespace Analyser;
  using namespace WNum::SAMURAIPLA;  

  TArtPlastic *pla;
  pla = calibsamuraipla->FindPlastic((char*)"F3pl");
  if(pla){
    f3pla_tl = pla->GetTLRaw();
    f3pla_tr = pla->GetTRRaw();
    if(pla->GetTLRaw() > 0 &&
       pla->GetTRRaw() > 0){
      f3pla_t = (f3pla_tl + f3pla_tr)/2;
      f3pla_dt = f3pla_tl - f3pla_tr;
    }
    if(pla->GetQLRaw() > 0 &&
       pla->GetQRRaw() > 0){
      //      f3pla_q = pla->GetQLRaw() + pla->GetQRRaw();
      f3pla_q = sqrt(pla->GetQLRaw() * pla->GetQRRaw());
      f3pla_lnq = log(pla->GetQLRaw()/pla->GetQRRaw());
    }

    if(!this->fAnaFile){
      hf3pla_tl->Fill(pla->GetTLRaw());
      hf3pla_tr->Fill(pla->GetTRRaw());
      hf3pla_ql->Fill(pla->GetQLRaw());
      hf3pla_qr->Fill(pla->GetQRRaw());
    }else{
      Add(SAMURAIPLA, ID,     1, 1);
      Add(SAMURAIPLA, QLRaw,  1, pla->GetQLRaw());
      Add(SAMURAIPLA, QRRaw,  1, pla->GetQRRaw());
      Add(SAMURAIPLA, QAveRaw,1, f3pla_q);  
      Add(SAMURAIPLA, LogQRaw,1, f3pla_lnq);   
      Add(SAMURAIPLA, TLRaw,  1, pla->GetTLRaw());
      Add(SAMURAIPLA, TRRaw,  1, pla->GetTRRaw());
      Add(SAMURAIPLA, TAveRaw,1, f3pla_t);
      Add(SAMURAIPLA, DTRaw,  1, f3pla_dt); 
    }
  }
  pla = calibsamuraipla->FindPlastic((char*)"F7pl");
  if(pla){
    f7pla_tl = pla->GetTLRaw();
    f7pla_tr = pla->GetTRRaw();
    if(pla->GetTLRaw() > 0 &&
       pla->GetTRRaw() > 0){
      f7pla_t = (f7pla_tl + f7pla_tr)/2;
      f7pla_dt = f7pla_tl - f7pla_tr;
    }
    if(pla->GetQLRaw() > 0 &&
       pla->GetQRRaw() > 0){
      //      f7pla_q = pla->GetQLRaw() + pla->GetQRRaw();
      f7pla_q = sqrt(pla->GetQLRaw() * pla->GetQRRaw());
      f7pla_lnq = log(pla->GetQLRaw()/pla->GetQRRaw());
    }

    if(!this->fAnaFile){
      hf7pla_tl->Fill(pla->GetTLRaw());
      hf7pla_tr->Fill(pla->GetTRRaw());
      hf7pla_ql->Fill(pla->GetQLRaw());
      hf7pla_qr->Fill(pla->GetQRRaw());
    }else{
      Add(SAMURAIPLA, ID,     3, 3);
      Add(SAMURAIPLA, QLRaw,  3, pla->GetQLRaw());
      Add(SAMURAIPLA, QRRaw,  3, pla->GetQRRaw());
      Add(SAMURAIPLA, QAveRaw,3, f7pla_q);  
      Add(SAMURAIPLA, LogQRaw,3, f7pla_lnq);   
      Add(SAMURAIPLA, TLRaw,  3, pla->GetTLRaw());
      Add(SAMURAIPLA, TRRaw,  3, pla->GetTRRaw());
      Add(SAMURAIPLA, TAveRaw,3, f7pla_t);
      Add(SAMURAIPLA, DTRaw,  3, f7pla_dt); 
    }
  }
  pla = calibsamuraipla->FindPlastic((char*)"F13pl-1");
  if(pla){
    f13pla1_tl = pla->GetTLRaw();
    f13pla1_tr = pla->GetTRRaw();
    if(pla->GetTLRaw() > 0 &&
       pla->GetTRRaw() > 0){
      f13pla1_t = (f13pla1_tl + f13pla1_tr)/2;
      f13pla1_dt = f13pla1_tl - f13pla1_tr;
    }
    if(pla->GetQLRaw() > 0 &&
       pla->GetQRRaw() > 0){
      //      f13pla1_q = pla->GetQLRaw() + pla->GetQRRaw();
      f13pla1_q = sqrt(pla->GetQLRaw() * pla->GetQRRaw());
      f13pla1_lnq = log(pla->GetQLRaw()/pla->GetQRRaw());
    }

    if(!this->fAnaFile){
      hf13pla1_tl->Fill(pla->GetTLRaw());
      hf13pla1_tr->Fill(pla->GetTRRaw());
      hf13pla1_ql->Fill(pla->GetQLRaw());
      hf13pla1_qr->Fill(pla->GetQRRaw());
    }else{
      Add(SAMURAIPLA, ID,     4, 4);
      Add(SAMURAIPLA, QLRaw,  4, pla->GetQLRaw());
      Add(SAMURAIPLA, QRRaw,  4, pla->GetQRRaw());
      Add(SAMURAIPLA, QAveRaw,4, f13pla1_q);  
      Add(SAMURAIPLA, LogQRaw,4, f13pla1_lnq);   
      Add(SAMURAIPLA, TLRaw,  4, pla->GetTLRaw());
      Add(SAMURAIPLA, TRRaw,  4, pla->GetTRRaw());
      Add(SAMURAIPLA, TAveRaw,4, f13pla1_t);
      Add(SAMURAIPLA, DTRaw,  4, f13pla1_dt); 
    }
  }
  pla = calibsamuraipla->FindPlastic((char*)"F13pl-2");
  if(pla){
    f13pla2_tl = pla->GetTLRaw();
    f13pla2_tr = pla->GetTRRaw();
    if(pla->GetTLRaw() > 0 &&
       pla->GetTRRaw() > 0){
      f13pla2_t = (f13pla2_tl + f13pla2_tr)/2;
      f13pla2_dt = f13pla2_tl - f13pla2_tr;
    }
    if(pla->GetQLRaw() > 0 &&
       pla->GetQRRaw() > 0){
      //      f13pla2_q = pla->GetQLRaw() + pla->GetQRRaw();
      f13pla2_q = sqrt(pla->GetQLRaw() * pla->GetQRRaw());
      f13pla2_lnq = log(pla->GetQLRaw()/pla->GetQRRaw());
    }

    if(!this->fAnaFile){
      hf13pla2_tl->Fill(pla->GetTLRaw());
      hf13pla2_tr->Fill(pla->GetTRRaw());
      hf13pla2_ql->Fill(pla->GetQLRaw());
      hf13pla2_qr->Fill(pla->GetQRRaw());
    }else{
      Add(SAMURAIPLA, ID,     5, 5);
      Add(SAMURAIPLA, QLRaw,  5, pla->GetQLRaw());
      Add(SAMURAIPLA, QRRaw,  5, pla->GetQRRaw());
      Add(SAMURAIPLA, QAveRaw,5, f13pla2_q);  
      Add(SAMURAIPLA, LogQRaw,5, f13pla2_lnq);   
      Add(SAMURAIPLA, TLRaw,  5, pla->GetTLRaw());
      Add(SAMURAIPLA, TRRaw,  5, pla->GetTRRaw());
      Add(SAMURAIPLA, TAveRaw,5, f13pla2_t);
      Add(SAMURAIPLA, DTRaw,  5, f13pla2_dt); 
    }
  }
  pla = calibsamuraipla->FindPlastic((char*)"SBV");
  if(pla){
    f13sbv_tl = pla->GetTLRaw();
    f13sbv_tr = pla->GetTRRaw();
    if(pla->GetTLRaw() > 0 &&
       pla->GetTRRaw() > 0){
      f13sbv_t = (f13sbv_tl + f13sbv_tr)/2;
      f13sbv_dt = f13sbv_tl - f13sbv_tr;
    }
    if(pla->GetQLRaw() > 0 &&
       pla->GetQRRaw() > 0){
      //      f13sbv_q = pla->GetQLRaw() + pla->GetQRRaw();
      f13sbv_q = sqrt(pla->GetQLRaw() * pla->GetQRRaw());
      f13sbv_lnq = log(pla->GetQLRaw()/pla->GetQRRaw());
    }

    if(!this->fAnaFile){
      hf13sbv_tl->Fill(pla->GetTLRaw());
      hf13sbv_tr->Fill(pla->GetTRRaw());
      hf13sbv_ql->Fill(pla->GetQLRaw());
      hf13sbv_qr->Fill(pla->GetQRRaw());
    }else{
      Add(SAMURAIPLA, ID,     6, 6);
      Add(SAMURAIPLA, QLRaw,  6, pla->GetQLRaw());
      Add(SAMURAIPLA, QRRaw,  6, pla->GetQRRaw());
      Add(SAMURAIPLA, QAveRaw,6, f13sbv_q);  
      Add(SAMURAIPLA, LogQRaw,6, f13sbv_lnq);   
      Add(SAMURAIPLA, TLRaw,  6, pla->GetTLRaw());
      Add(SAMURAIPLA, TRRaw,  6, pla->GetTRRaw());
      Add(SAMURAIPLA, TAveRaw,6, f13sbv_t);
      Add(SAMURAIPLA, DTRaw,  6, f13sbv_dt); 
    }
  }

  //  if(calibsamuraipla->FindPlastic("F13pl-1") &&
  //
  //  calibsamuraipla->FindPlastic("F13pl-2")){
    if(f13pla2_tl > 0 && 
       f13pla2_tr > 0 &&
       f13pla1_tl > 0 &&    
       f13pla1_tr > 0){
      Double_t ftof1313 = f13pla2_t - f13pla1_t;
      if(!this->fAnaFile){
	/*nothing*/
      }else{
	//	std::cout << ftof1313 << std::endl;
	Add(SAMURAIPLA, TOF1313, 0, ftof1313);
      }	
    }
    //  }

  if(!this->fAnaFile){
    ntp->Fill((float)f3pla_tl,(float)f3pla_tr,(float)f3pla_q,
	      (float)f7pla_tl,(float)f7pla_tr,(float)f7pla_q,
	      (float)f13pla1_tl,(float)f13pla1_tr,(float)f13pla1_q,
	      (float)f13pla2_tl,(float)f13pla2_tr,(float)f13pla2_q,
	      (float)f13sbv_tl,(float)f13sbv_tr,(float)f13sbv_q);
  }
}

void TAlSAMURAIPlaExample::Destruct()
{
  delete calibsamuraipla;
  calibsamuraipla = 0; 
  //  delete samuraiparameters;
  samuraiparameters->Delete();
  samuraiparameters = 0;
}

const char* TAlSAMURAIPlaExample::ClassName() const
{
  return "TAlSAMURAIPlaExample";
}
