
#include "TArtAnaLoop.hh"

#include <iostream>

#include "TThread.h"
#include "TObject.h"
#include "TSystem.h"

#include "TArtEventStore.hh"
#include "TArtStoreManager.hh"

#include "TArtAnaLoopManager.hh"
#include "TArtAnaFile.hh"
#include "TArtAnaClock.hh"
#include "EAnalyser.hh"

#include <vector>

TArtAnaLoop::TArtAnaLoop()
  : fAnaFile(0), fEventStore(0), fNumEvents(0), fNumTotalEvents(0)
{
  for(int i=0; i<TArtValMap::MaxAnalyserSize; ++i) fAnalyserFlag[i] = true;
}

TArtAnaLoop::~TArtAnaLoop()
{
  delete fAnaFile;
  fAnaFile = 0;
  delete fEventStore;
  fEventStore = 0;
  delete TArtStoreManager::Instance();
}

bool TArtAnaLoop::OpenAnaFile(const char* anafilename)
{
  if(fAnaFile){
    std::cout << "destruct current anafile" << std::endl;
    delete fAnaFile;
  }
  fAnaFile = new TArtAnaFile;
  if(!fAnaFile->Construct() ||
     !fAnaFile->Open(anafilename) ||
     !fAnaFile->Check()){
    std::cout << "anafile error" << std::endl;
    delete fAnaFile;
    fAnaFile = 0;
    return false;
  }

  SetAnalyser(fAnaFile->GetAnalyser());
  return true;
}

bool TArtAnaLoop::ChangeAnaFile(const char* anafilename)
{
  if(!fAnaFile){
    std::cout << "no anafile is opened. construct new anafile." << std::endl;
    return OpenAnaFile(anafilename);
  }

  fAnaFile->ClearVal();
  fAnaFile->ClearMap();
  if(!fAnaFile->Construct() ||
     !fAnaFile->Open(anafilename) ||
     !fAnaFile->Check()){
    std::cout << "anafile error" << std::endl;
    return false;
  }

  SetAnalyser(fAnaFile->GetAnalyser());
  return true;
}

void TArtAnaLoop::AnaLoop(const char* filename, int eventnumber)
{
  int sid = 0;

  while(1){
    if(TArtAnaLoopManager::kAnalysisState == TArtAnaLoopManager::INIT){
      TArtAnaLoopManager::fLock = true;

      Construct();
      TArtAnaLoopManager::fLock = false;

      TArtAnaLoopManager::kAnalysisState = TArtAnaLoopManager::NEXT;
    }

    while(TArtAnaLoopManager::kAnalysisState == TArtAnaLoopManager::NEXT){
      static bool init = true;
      if(TArtAnaLoopManager::fNextRidfFileName.empty()){
	if(init) std::cout << "\n waiting for next data file\n" << std::endl;    
	init = false;
	if(TArtAnaLoopManager::kAnalysisState == TArtAnaLoopManager::END){
	  init = true;
	  break;
	}
	if(gSystem) gSystem->Sleep(500);
	while(TArtAnaLoopManager::kAnalysisState == TArtAnaLoopManager::SUSPENDED){
	  if(gSystem) gSystem->Sleep(500);
	}
	continue;
      }
      init = true;

      const char* filename_next = TArtAnaLoopManager::fNextRidfFileName[0];
      int eventnumber_next = TArtAnaLoopManager::fNextRidfEventNumber[0];
      int sid_next = TArtAnaLoopManager::fNextSID[0];
      TArtAnaLoopManager::fNextRidfFileName.pop_front();
      TArtAnaLoopManager::fNextRidfEventNumber.pop_front();
      TArtAnaLoopManager::fNextSID.pop_front();
      filename = filename_next;
      eventnumber = eventnumber_next;
      sid = sid_next;
      TArtAnaLoopStatic::SetFileName(filename);
      fNumEvents = 0;
      TArtAnaClock::Reset();

      TArtAnaLoopManager::fLock = true;

      if(!fEventStore){
	fEventStore = new TArtEventStore;
	fEventStore->SetInterrupt(&TArtAnaLoopManager::fStopLoop);
      }
      bool isopen;
      if(!filename || !strcmp(filename, "")){
	isopen = fEventStore->Open(sid);
      }else{
	isopen = fEventStore->Open(filename);
      }
      if(!isopen){
	delete fEventStore;
	fEventStore = 0;
	TArtAnaLoopManager::kAnalysisState = TArtAnaLoopManager::ERROR;

	TArtAnaLoopManager::fLock = false;
	break;
      }
      TArtAnaLoopManager::fLock = false;
      TArtAnaLoopManager::kAnalysisState = TArtAnaLoopManager::RUNNING;
      break;
    }

    if(TArtAnaLoopManager::kAnalysisState == TArtAnaLoopManager::RUNNING){
      while(TArtAnaLoopManager::kAnalysisState == TArtAnaLoopManager::RUNNING){
	if(!(eventnumber<0) && fNumEvents>eventnumber-1){
	  std::cout << "\n\n analysis completed!! \n" << std::endl;
	  TArtAnaLoopManager::kAnalysisState = TArtAnaLoopManager::NEXT;
	  break;
	}

 	bool nextevent = false;
	TArtAnaLoopManager::fLock = true;
	TArtAnaLoopManager::fStopLoop = false;
	nextevent = fEventStore->GetNextEvent();
	TArtAnaLoopManager::fLock = false;
	if(!filename || !strcmp(filename, "")){//online
//	  while(!nextevent){
//	    if(TArtAnaLoopManager::kAnalysisState != TArtAnaLoopManager::RUNNING) break;
//	    else WaitOnlineEvent();
//	    TArtAnaLoopManager::fLock = true;
//	    TArtAnaLoopManager::fStopLoop = false;
//	    nextevent = fEventStore->GetNextEvent();
//	    TArtAnaLoopManager::fLock = false;
//	  }
//	  if(!nextevent) break;
	  if(!nextevent){
	    while(TArtAnaLoopManager::kAnalysisState == TArtAnaLoopManager::SUSPENDED){
	      if(gSystem) gSystem->Sleep(500);
	    }
	    continue;
	  }
	}else{//offline
	  if(!nextevent){
	    if(TArtAnaLoopManager::fStopLoop == true){
	      if(gSystem) gSystem->Sleep(500);
	      continue;
	    }
	    std::cout << "\n\n analysis completed!! \n" << std::endl;
	    TArtAnaLoopManager::kAnalysisState = TArtAnaLoopManager::NEXT;
	    break;
	  }
	}

	TArtAnaLoopManager::fLock = true;
	++fNumEvents;
	++fNumTotalEvents;

	TArtAnaClock::Start();

	if(fAnaFile) fAnaFile->ClearVal();
	Calculate();
	if(fAnaFile) fAnaFile->Fill();
	fEventStore->ClearData();

	TArtAnaClock::Stop();

	TArtAnaLoopManager::fLock = false;

	while(TArtAnaLoopManager::kAnalysisState == TArtAnaLoopManager::SUSPENDED){
	  if(gSystem) gSystem->Sleep(500);
	}
      }
      TArtAnaLoopManager::fPrevRidfFileName.push_back(filename);
      TArtAnaLoopManager::fPrevRidfEventNumber.push_back(fNumEvents);
      TArtAnaLoopManager::fPrevSID.push_back(sid);
    }

    if(TArtAnaLoopManager::kAnalysisState == TArtAnaLoopManager::NEXT) continue;

    if(TArtAnaLoopManager::kAnalysisState == TArtAnaLoopManager::END){
      TArtAnaLoopManager::fLock = true;

      Destruct();
      delete fEventStore;
      fEventStore = 0;
      delete TArtStoreManager::Instance();

      TArtAnaLoopManager::fLock = false;
      break;
    }

    std::cout << "nevert to come here(kAnalysisState: "
	      << TArtAnaLoopManager::kAnalysisState << ")" << std::endl;
    TArtAnaLoopManager::kAnalysisState = TArtAnaLoopManager::ERROR;
    break;
  }

  if(TArtAnaLoopManager::kAnalysisState == TArtAnaLoopManager::ERROR){
    std::cout << "\n Some error occur. Call \"book() or clear()\" to Recover. \n" << std::endl;    
  }
}

bool TArtAnaLoop::Add(int analyser, int wnum, int index, double val)
{
  if(fAnaFile) return fAnaFile->Add(analyser, wnum, index, val);
  else return false;
}

void TArtAnaLoop::SetAnalyser(std::vector<int> analyser_v)
{
  if(analyser_v.empty()){
    for(int i=0; i<TArtValMap::MaxAnalyserSize; ++i) fAnalyserFlag[i] = true;
  }else{
    for(int i=0; i<TArtValMap::MaxAnalyserSize; ++i) fAnalyserFlag[i] = false;    
  }
}

void TArtAnaLoop::SetAnalyserFlag(std::vector<int> analyser_v, int analyser)
{
  for(int i=0; i<(int)analyser_v.size(); ++i){
    if(analyser_v[i] == analyser){
      fAnalyserFlag[analyser_v[i]] =  true;
      //      std::cout << analyser << std::endl;
    }
  }
}

void TArtAnaLoop::WaitOnlineEvent()
{
  gSystem->Sleep(500);
}
