// handler of BigRIPS parameters

#ifndef _TARTBIGRIPSPARAMETERS_H_
#define _TARTBIGRIPSPARAMETERS_H_

#include <TROOT.h>
#include <TList.h>
#include <map>

class TXMLNode;
class TArtPPACPara;
class TArtPlasticPara;
class TArtICPara; 
class TArtFocalPlanePara; 
class TArtRIDFMap;

class TArtBigRIPSParameters : public TNamed
{
 public: 

  TArtBigRIPSParameters(const char* name = "BigRIPSParameters", 
			const char* title = "BigRIPSParameters");
  ~TArtBigRIPSParameters();
  static TArtBigRIPSParameters* Instance(const char* name = "BigRIPSParameters", 
					 const char* title = "BigRIPSParameters");
  static void Delete();

  Bool_t LoadParameter(char *xmlfile);

  void PrintListOfPPACPara();
  void PrintListOfPlasticPara();
  void PrintListOfICPara();
  void PrintListOfFocalPlanePara();

  TList* GetListOfPPACPara()      {return listOfPPACPara; }
  TList* GetListOfPlasticPara()   {return listOfPlasticPara; }
  TList* GetListOfICPara()        {return listOfICPara; }
  TList* GetListOfFocalPlanePara(){return listOfFocalPlanePara;}

  Int_t GetNumPPACPara()      {return listOfPPACPara->GetEntries(); }
  Int_t GetNumPlasticPara()   {return listOfPlasticPara->GetEntries(); }
  Int_t GetNumICPara()        {return listOfICPara->GetEntries(); }
  Int_t GetNumFocalPlanePara(){return listOfFocalPlanePara->GetEntries();}

  TArtPPACPara       * FindPPACPara(TArtRIDFMap *rmap);
  TArtPlasticPara    * FindPlasticPara(TArtRIDFMap *rmap);
  TArtPlasticPara    * FindPlasticPara(TString *name);
  TArtICPara         * FindICPara(TArtRIDFMap *rmap);
  TArtFocalPlanePara * FindFocalPlanePara(int fpl);

  void   SetFocusPosOffset(Int_t fpl, Double_t offset);

 protected:

  std::map<TArtRIDFMap, TArtPPACPara *> ppac_pmap;
  std::map<TArtRIDFMap, TArtPlasticPara *> pla_pmap;
  std::map<TString, TArtPlasticPara *> pla_name_pmap;
  std::map<TArtRIDFMap, TArtICPara *> ic_pmap;
  std::map<int, TArtFocalPlanePara *> fpl_pmap;

  TList *listOfPPACPara;
  TList *listOfPlasticPara;
  TList *listOfICPara;
  TList *listOfFocalPlanePara;

  void ParseParaList(TXMLNode *node);
  TArtPPACPara * ParsePPACPara(TXMLNode *node);
  TArtPlasticPara * ParsePlasticPara(TXMLNode *node);
  TArtICPara * ParseICPara(TXMLNode *node);
  TArtFocalPlanePara * ParseFocalPlanePara(TXMLNode *node);

  static TArtBigRIPSParameters* fBigRIPSParameters;

  ClassDef(TArtBigRIPSParameters,1);

};

#endif
