// Drift Chamber Track reconstruction class
// 

#ifndef _TARTCALIBDCTRACK_H_
#define _TARTCALIBDCTRACK_H_

#include <TClonesArray.h>
#include <TVectorD.h>
#include <TH1.h>
#include <vector>

#include "TArtReconstruction.hh"

class TArtDCTrack;
class TArtDCHit;
class TArtSAMURAIParameters;
class TArtStoreManager;

class TArtCalibDCTrack : public TArtReconstruction {

 public:

  TArtCalibDCTrack(const char *indata, const char *outdata, const char *lpattern);
  virtual ~TArtCalibDCTrack();

  virtual void ClearData();
  virtual void ReconstructData();

  // function to set tdc distribution for drift length calculation.
  virtual void SetTDCDistribution(TH1 * inhtdc);

  // function to access data container
  TClonesArray    * GetDCTrackArray(){return fDCTrackArray;}
  Int_t             GetNumDCTrack();
  Int_t             GetNumDCHit(int l);
  TArtDCTrack     * GetDCTrack(int i);

 private:
  // 1d track (x,y,u or v) buffer, which will be combined to make 1 track
  class Track1D : public TObject {
   public:
    Track1D(){x=0;a=0;chi2=99999.;ndf=-1;hitx=NULL;hitz=NULL;}
    ~Track1D(){if(hitx) delete hitx; if(hitz) delete hitz;}
    Track1D& operator=(const Track1D &right);
    Double_t x;
    Double_t a;
    Double_t chi2;
    Int_t ndf;
    Int_t nhit;
    Double_t *hitx;
    Double_t *hitz;

    // overriding functions for sorting based on chi2
    Bool_t IsEqual(TObject *obj) const {return chi2 == ((Track1D*)obj)->chi2;}
    Bool_t IsSortable() const { return kTRUE; } 
    Int_t Compare(const TObject *obj) const { 
      if (chi2 < ((Track1D*)obj)->chi2) return -1; 
      else if (chi2 > ((Track1D*)obj)->chi2) return 1; 
      else return 0; 
    }
  };

 protected:

  TString indataname;

  // calculate drift length on the assumption of x=t*dv
  Double_t CalcDriftLength(Double_t t);
  Double_t tdczero; // tdczero for dl calculation: dl=(tdczero-tdc)*driftv
  Double_t driftv; // mm/tdcch
  Double_t pitch; // mm

  // parameters for calculation of drift length by using tdc distribution
  Int_t tdcwindow[2]; // tdc bin window for dl calc.
  Double_t tdcint; // tdc integral for dl calc.
  TH1 * htdc; // tdc distribution for tdc <-> dl conv.

  void SetHitsBuffer();

  void RecoSimpleTrack(std::vector <TArtDCHit *> *hits, Int_t nl, Int_t dir);
  void CalcSimpleTrack(Double_t pos[][2], Int_t nl, TArtDCTrack *tr, Int_t dir); // dir=0 means tracking for x and 1 means for y 
  void FindAssociatePos(TArtDCTrack *tr, std::vector <TArtDCHit *> *hits, Double_t *pos);

  void RecoTrack();
  void CalcTrack(std::vector <TArtDCHit *> *hits, Int_t numlayer, TObjArray *trs);
  Track1D * CalcTrack1D(std::vector <TArtDCHit *> *hits);
  TArtDCTrack * CalcTrackWith1DTracks(Track1D *tx, Track1D *tu, Track1D *tv);

 private:

  TString layer_pattern;
  Int_t nlayer, nlayer_x, nlayer_y, nlayer_u, nlayer_v;
  std::vector <int> layer_map; // map for global layer -> local (x,y,u or v) layer

  TClonesArray     * fDCTrackArray;

  TArtStoreManager * sman;

  // buffer to making tracks
  std::vector <TArtDCHit *> *hits_buffer;
  std::vector <TArtDCHit *> *hits_buffer_x;
  std::vector <TArtDCHit *> *hits_buffer_y;
  std::vector <TArtDCHit *> *hits_buffer_u;
  std::vector <TArtDCHit *> *hits_buffer_v;

  ClassDef(TArtCalibDCTrack,1)

};

#endif
