// NEBULA calibration class
// 

#ifndef _TARTCALIBNEBULA_H_
#define _TARTCALIBNEBULA_H_

#include <TNamed.h>
#include <vector>
#include <map>

#include "TArtReconstruction.hh"

#include "TArtRawEventObject.hh"
#include "TArtCore.hh" 
#include "TArtStoreManager.hh"
#include "TArtMapper.hh"
#include "segidlist.hh"

#include "TArtSAMURAIParameters.hh"
#include "TArtNEBULAPla.hh"
#include "TArtNEBULAPlaPara.hh"

#include <TClonesArray.h>
#include <iostream>
#include <cmath>

class TClonesArray;
class TArtStoreManager;
class TArtSAMURAIParameters;

class TArtNEBULAPla;
typedef TClonesArray TArtNEBULAPlaArray;
class TArtNEBULAPlaPara;
typedef TClonesArray TArtNEBULAPlaParaArray;

class TString;

class TArtCalibNEBULANeutron : public TObject
{
public:
  TArtCalibNEBULANeutron();
  ~TArtCalibNEBULANeutron();

  void ClearData();

  Double_t fQ;
  Double_t fT;
  Double_t fBeta[3];
  Double_t fGamma[3];
  Double_t fP[3];
  Double_t fPabs;
  Double_t fE;
  Double_t fTheta;
  Double_t fPhi;

  ClassDef(TArtCalibNEBULANeutron,1)
};

class TArtCalibNEBULAMult : public TObject
{
public:
  TArtCalibNEBULAMult();
  ~TArtCalibNEBULAMult();

  void ClearData();

  Int_t fMultiplicity_N;
  Int_t fMultiplicity_N1;
  Int_t fMultiplicity_N2;
  Int_t fMultiplicity_V1;
  Int_t fMultiplicity_V2;
  Int_t fMultiplicity_N11;
  Int_t fMultiplicity_N12;
  Int_t fMultiplicity_N21;
  Int_t fMultiplicity_N22;

  ClassDef(TArtCalibNEBULAMult,1)
};

class TArtCalibNEBULA : public TArtReconstruction
{
public:
  TArtCalibNEBULA();
  virtual ~TArtCalibNEBULA();

  void LoadRawData();
  void ReconstructData();
  void ClearData();

  Int_t GetNumNEBULAPla() const;
  TArtNEBULAPla* GetNEBULAPla(int i) const;
  TArtNEBULAPla* FindNEBULAPla(int id) const;//find by id
  TArtNEBULAPlaArray* GetNEBULAPlaArray() const;
  Int_t GetNumNEBULAPlaPara() const;
  TArtNEBULAPlaPara* GetNEBULAPlaPara(int i) const;
  TArtNEBULAPlaPara* FindNEBULAPlaPara(int id) const;//find by id
  TArtNEBULAPlaParaArray* GetNEBULAPlaParaArray() const;

  void  CalMultiplicity(); // calculate multiplicity
  void  CalNeutron(); // calculate neutron
  
  Int_t GetMultiplicity_N(){  return fMult.fMultiplicity_N  ;}
  Int_t GetMultiplicity_N1(){ return fMult.fMultiplicity_N1 ;}
  Int_t GetMultiplicity_N2(){ return fMult.fMultiplicity_N2 ;}
  Int_t GetMultiplicity_V1(){ return fMult.fMultiplicity_V1 ;}
  Int_t GetMultiplicity_V2(){ return fMult.fMultiplicity_V2 ;}
  Int_t GetMultiplicity_N11(){return fMult.fMultiplicity_N11;}
  Int_t GetMultiplicity_N12(){return fMult.fMultiplicity_N12;}
  Int_t GetMultiplicity_N21(){return fMult.fMultiplicity_N21;}
  Int_t GetMultiplicity_N22(){return fMult.fMultiplicity_N22;}

  TArtCalibNEBULANeutron* GetNeutron(){return &fNeutron;} 
  TArtCalibNEBULAMult* GetMultiplicities(){return &fMult;} 
 
private:

  TArtNEBULAPlaArray* fNEBULAPlaArray;
  TArtNEBULAPlaParaArray* fNEBULAPlaParaArray;

  const TArtSAMURAIParameters* fSAMURAIParameters;

  TArtStoreManager* fStoreManager;

  TArtCalibNEBULANeutron fNeutron;
  TArtCalibNEBULAMult fMult;

  //for optimization
  std::map<int,int> fIDNPlaMap;
  std::map<int,int> fIDNPlaParaMap;

  //only for sm_com (where should store like these data?)
  /*
    modified to simplify reference to ROOT/TTree :
    in principle data for every plastic is with a TClonesArray
    these data are hence outside. 
    they should be kept here within this class (easier to find)
    ## Julien 20 March 2012 ##
  */ 
  //int fMultiplicity_N;
  //int fMultiplicity_N1,  fMultiplicity_N2;
  //int fMultiplicity_V1,  fMultiplicity_V2;
  //int fMultiplicity_N11, fMultiplicity_N12;
  //int fMultiplicity_N21, fMultiplicity_N22;


  ClassDef(TArtCalibNEBULA,1);
};

#endif
