#ifndef _TARTDCTRACK_H_
#define _TARTDCTRACK_H_

#include "TArtCore.hh"
#include "TArtDataObject.hh"

//
// DC Track Data Container
//
// In the case of combined tracking of X,U and V layer, both x and y are supposed to be filled. 
// Otherwise part of x or y will be filled.
// 

class TArtDCTrack : public TArtDataObject {

 public:
  TArtDCTrack(){nhitlayer=0;chi2=0;ndf=0;for(int i=0;i<2;i++){x[i]=-9999;a[i]=-9999;}}
  virtual ~TArtDCTrack(){}

  // function to set data
  virtual void SetNumHitLayer(Int_t val){nhitlayer = val;}
  virtual void SetChi2(Double_t val){chi2 = val;}
  virtual void SetNDF(Int_t val){ndf = val;}
  virtual void SetPosition(Double_t val, int i=0){x[i] = val;}
  virtual void SetAngle(Double_t val, int i=0){a[i] = val;}

  // function to get data
  virtual Int_t GetNumHitLayer(){return nhitlayer;}
  virtual Double_t GetChi2(){return chi2;}
  virtual Int_t GetNDF(){return ndf;}
  virtual Double_t GetPosition(int i=0){return x[i];}
  virtual Double_t GetAngle(int i=0){return a[i];}

  virtual void Print(){
    TArtCore::Info(__FILE__,"nhit:%d, chi2:%f, ndf:%d, x:%f, a:%f, y:%f, b:%f",
		   nhitlayer,chi2,ndf,x[0],a[0],x[1],a[1]);
  }

 private:

  // parameters
  Int_t nhitlayer; // number of hit layer
  Double_t chi2; // track fit chi2 
  Int_t ndf; // track fit ndf

  Double_t x[2]; // track position, 0: x, 1: y 
  Double_t a[2]; // track angle,  0: x, 1: y 

 public:
  // overriding functions for sorting based on chi2
  Bool_t IsEqual(TObject *obj) const { return chi2 == ((TArtDCTrack*)obj)->chi2;} 
  Bool_t IsSortable() const { return kTRUE; } 
  Int_t Compare(const TObject *obj) const { 
    if (chi2 < ((TArtDCTrack*)obj)->chi2) return -1; 
    else if (chi2 > ((TArtDCTrack*)obj)->chi2) return 1; 
    else return 0; 
  }

  ClassDef(TArtDCTrack,1)

};
#endif
