#ifndef _TARTNEBULAPARAMETERS_H_
#define _TARTNEBULAPARAMETERS_H_

#include <TNamed.h>
#include <TList.h>

#include <map>

class TXMLNode;
class TArtRIDFMap;
class TArtStoreManager;
class TArtICPara;
class TArtPlasticPara;
class TArtDCHitPara;
class TArtHODPlaPara;
class TArtNEBULAPlaPara;

typedef std::map<TArtRIDFMap, const TArtNEBULAPlaPara*> TArtNEBULAPlaParaMap;
typedef std::pair<TArtRIDFMap, const TArtNEBULAPlaPara*> TArtNEBULAPlaParaPair;
class TArtNEBULAHPCPara;
typedef std::map<TArtRIDFMap, const TArtNEBULAHPCPara*> TArtNEBULAHPCParaMap;
typedef std::pair<TArtRIDFMap, const TArtNEBULAHPCPara*> TArtNEBULAHPCParaPair;

class TArtSAMURAIParameters : public TNamed
{
public: 
  TArtSAMURAIParameters(const char* name = "SAMURAIParameters", 
		       const char* title = "SAMURAIParameters");
  ~TArtSAMURAIParameters();
  static TArtSAMURAIParameters* Instance(const char* name = "SAMURAIParameters", 
					 const char* title = "SAMURAIParameters");
  static void Delete();

  Bool_t LoadParameter(const char *xmlfile);

  void PrintListOfPlasticPara ();
  void PrintListOfICPara ();
  void PrintListOfHODPlaPara ();
  void PrintListOfBPCHitPara();
  void PrintListOfBDC1HitPara();
  void PrintListOfBDC2HitPara();
  void PrintListOfFDC1HitPara();
  void PrintListOfFDC2HitPara();
  void PrintListOfNEBULAPlaPara() const;

  TList* GetListOfPlasticPara() {return listOfPlasticPara; }
  TList* GetListOfICPara    () {return listOfICPara; }
  TList* GetListOfHODPlaPara () {return listOfHODPlaPara; }
  TList* GetListOfBPCHitPara () {return listOfBPCHitPara; }
  TList* GetListOfBDC1HitPara() {return listOfBDC1HitPara; }
  TList* GetListOfBDC2HitPara() {return listOfBDC2HitPara; }
  TList* GetListOfFDC1HitPara() {return listOfFDC1HitPara; }
  TList* GetListOfFDC2HitPara() {return listOfFDC2HitPara; }

  const TArtPlasticPara* FindPlasticPara (TArtRIDFMap *rmap) const;
  const TArtICPara     * FindICPara     (TArtRIDFMap *rmap) const;
  const TArtHODPlaPara* FindHODPlaPara (TArtRIDFMap *rmap) const;
  const TArtDCHitPara * FindBPCHitPara (TArtRIDFMap *rmap) const;
  const TArtDCHitPara * FindBDC1HitPara(TArtRIDFMap *rmap) const;
  const TArtDCHitPara * FindBDC2HitPara(TArtRIDFMap *rmap) const;
  const TArtDCHitPara * FindFDC1HitPara(TArtRIDFMap *rmap) const;
  const TArtDCHitPara * FindFDC2HitPara(TArtRIDFMap *rmap) const;

  const TArtNEBULAPlaPara* FindNEBULAPlaUPara(const TArtRIDFMap &rmap) const;
  const TArtNEBULAPlaPara* FindNEBULAPlaDPara(const TArtRIDFMap &rmap) const;
  const TArtNEBULAPlaPara* FindNEBULAPlaPara(int id) const;//find by id

  const TArtNEBULAHPCPara* FindNEBULAHPCPara(const TArtRIDFMap &rmap) const;
  const TArtNEBULAHPCPara* FindNEBULAHPCPara(int id) const;//find by id

private:
  void ParseParaList(TXMLNode* node);
  TArtDCHitPara* ParseDCHitPara(TXMLNode* node, std::map<TArtRIDFMap, TArtDCHitPara *> * map);
  TArtPlasticPara*   ParsePlasticPara(TXMLNode* node);
  TArtICPara*        ParseICPara(TXMLNode* node);
  TArtHODPlaPara*    ParseHODPlaPara(TXMLNode* node);
  TArtNEBULAPlaPara* ParseNEBULAPlaPara(TXMLNode* node);
  TArtNEBULAHPCPara* ParseNEBULAHPCPara(TXMLNode* node);

  std::map<TArtRIDFMap, TArtPlasticPara *> pla_pmap;
  std::map<TArtRIDFMap, TArtICPara *> icb_pmap;
  std::map<TArtRIDFMap, TArtHODPlaPara *> hod_pmap;
  std::map<TArtRIDFMap, TArtDCHitPara *> bpc_pmap;
  std::map<TArtRIDFMap, TArtDCHitPara *> bdc1_pmap;
  std::map<TArtRIDFMap, TArtDCHitPara *> bdc2_pmap;
  std::map<TArtRIDFMap, TArtDCHitPara *> fdc1_pmap;
  std::map<TArtRIDFMap, TArtDCHitPara *> fdc2_pmap;

  TArtNEBULAPlaParaMap *fNEBULAPlaUParaMap, *fNEBULAPlaDParaMap;
  TArtNEBULAHPCParaMap *fNEBULAHPCParaMap;

  // list of parameters 
  TList *listOfPlasticPara;
  TList *listOfICPara;
  TList *listOfHODPlaPara;
  TList *listOfBPCHitPara;
  TList *listOfBDC1HitPara;
  TList *listOfBDC2HitPara;
  TList *listOfFDC1HitPara;
  TList *listOfFDC2HitPara;

  // for optimization
  std::map<int, TArtNEBULAPlaPara*> fIDNEBULAPlaParaMap;
  std::map<int, TArtNEBULAHPCPara*> fIDNEBULAHPCParaMap;

  TArtStoreManager* fStoreManager;
  static TArtSAMURAIParameters* fSAMURAIParameters;

  ClassDef(TArtSAMURAIParameters,1);
};

#endif
