#include "TArtCalibDCHit.hh" 
#include "TArtCalibBDC1Hit.hh" 
#include "TArtDCHit.hh"
#include "TArtDCHitPara.hh"
#include "TArtSAMURAIParameters.hh"
#include "TArtRawEventObject.hh"
#include "TArtCore.hh" 
#include "segidlist.hh" 

#include <TROOT.h>

//__________________________________________________________
TArtCalibBDC1Hit::TArtCalibBDC1Hit() : TArtCalibDCHit("SAMURAIBDC1Hit", BDC)
{
  TArtCore::Info(__FILE__,"Creating the BDC1 detector hit objects...");
}

//__________________________________________________________
TArtCalibBDC1Hit::~TArtCalibBDC1Hit()  {
}

//__________________________________________________________
void TArtCalibBDC1Hit::LoadData(TArtRawSegmentObject *seg)   {

  int device   = seg->GetDevice();
  int fpl      = seg->GetFP();
  int detector = seg->GetDetector();
  if(detectorid != detector) return;
  for(int j=0;j<seg->GetNumData();j++){
    TArtRawDataObject *d = seg->GetData(j);
    int geo = d->GetGeo(); 
    int ch = d->GetCh(); 
    int edge = d->GetEdge(); 
    if(0!=edge) continue;
    unsigned int val = d->GetVal();
    TArtRIDFMap mm(fpl,detector,geo,ch);
    TArtDCHitPara *para = (TArtDCHitPara*)setup->FindBDC1HitPara(&mm);
    if(NULL == para){
      TArtCore::Debug(__FILE__,"Could not find TArtDCHitPara...: Dev:%d, %s", device, mm.GetMapInfo());
      continue;
    }
    else {
      TArtCore::Debug(__FILE__,"Find TArtDCHitPara...: Dev:%d, %s, : %s", device, mm.GetMapInfo(), para->GetDetectorName()->Data());
    }

    int id = para->GetID();
    int nhit = fDCHitArray->GetEntries();
    TArtCore::Debug(__FILE__,"Entries in DCHitArray: %d", nhit);
    TArtDCHit * hit = FindDCHit(id);

    if(NULL==hit){
      new ((*fDCHitArray)[nhit]) TArtDCHit();
      hit = (TArtDCHit *)fDCHitArray->At(nhit);
      hit->SetID(id);
      fDCHitParaArray.push_back(para);
    }

    // set raw data
    if(detectorid == detector){
      if(mm==*((TArtRIDFMap *)para->GetTDCMap()))
	hit->SetRealTDC(val);
    }

  }

  fDataLoaded = true;
  return;
}

