#include "TArtCalibCoin.hh" 
#include "TArtRawEventObject.hh"
#include "TArtEventInfo.hh"
#include "TArtCore.hh" 
#include "TArtStoreManager.hh"

#include "segidlist.hh"

#include <TROOT.h>

//__________________________________________________________
TArtCalibCoin::TArtCalibCoin() : TArtReconstruction("EventInfo") {
  TArtCore::Info(__FILE__,"Creating the Coincidence register calibration objects...");
  sman = TArtStoreManager::Instance();

  // event info class is not registered to StoreManager since it is supposed to be made in TArtEventStore
}

//__________________________________________________________
TArtCalibCoin::~TArtCalibCoin()  {
}

//__________________________________________________________
void TArtCalibCoin::LoadData()   {

  TArtRawEventObject *fEvent = (TArtRawEventObject *)sman->FindDataContainer("RawEvent");
  for(int i=0;i<fEvent->GetNumSeg();i++){
    TArtRawSegmentObject *seg = fEvent->GetSegment(i);
    int detector = seg->GetDetector();
    if(COIN == detector)
      LoadData(seg);
  }
 
  return;

}

//__________________________________________________________
void TArtCalibCoin::LoadData(TArtRawSegmentObject *seg)   {

  int detector = seg->GetDetector();

  if(COIN != detector) return;
  if(seg->GetNumData()>1) 
    TArtCore::Error(__FILE__,"Can not handle more than two data.:%d",seg->GetNumData());
  TArtRawDataObject *d = seg->GetData(0);

  val = d->GetVal();
  TClonesArray *info = (TClonesArray*)sman->FindDataContainer("EventInfo");
  ((TArtEventInfo *)info->At(0))->SetTriggerBit(val);

  fDataLoaded = true;
  return;

}

//__________________________________________________________
bool TArtCalibCoin::IsChTrue(Int_t id) {
  return ((1 << (id - 1)) & val) ? true : false;
}
