#include "TArtCalibIC.hh" 
#include "TArtCalibICB.hh" 
#include "TArtIC.hh"
#include "TArtICPara.hh"
#include "TArtSAMURAIParameters.hh"
#include "TArtRawEventObject.hh"
#include "TArtStoreManager.hh"
#include "TArtCore.hh" 
#include "segidlist.hh" 

#include <TROOT.h>

//__________________________________________________________
TArtCalibICB::TArtCalibICB() : TArtCalibIC("SAMURAIICB")
{
  TArtCore::Info(__FILE__,"Creating the SAMURAI-ICB detector objects...");
}

//__________________________________________________________
TArtCalibICB::~TArtCalibICB()  {
}

//__________________________________________________________
void TArtCalibICB::LoadData(TArtRawSegmentObject *seg)   {

  TArtSAMURAIParameters *setup = (TArtSAMURAIParameters*)sman->FindParameters("SAMURAIParameters");
  if(!setup){
    TArtCore::Info(__FILE__,"Can not find parameter handler: SAMURAIParameters");
    throw;
  }

  int device   = seg->GetDevice();
  int fpl      = seg->GetFP();
  int detector = seg->GetDetector();
  if(device != SAMURAI) return;
  if(ICB != detector) return;

  for(int j=0;j<seg->GetNumData();j++){
    TArtRawDataObject *d = seg->GetData(j);
    int geo = d->GetGeo(); 
    int ch = d->GetCh(); 
    unsigned int val = d->GetVal();
    TArtRIDFMap mm(fpl,detector,geo,ch);
    TArtICPara *para = (TArtICPara*)setup->FindICPara(&mm);
    if(NULL == para){
      TArtCore::Warning(__FILE__,"Could not find TArtICPara...: Dev:%d, %s", device, mm.GetMapInfo());
      continue;
    }
    else {
      TArtCore::Debug(__FILE__,"Find TArtICPara...: Dev:%d, %s, : %s", device, mm.GetMapInfo(), para->GetDetectorName()->Data());
    }

    int id = para->GetID();
    int nic = fICArray->GetEntries();
    TArtCore::Debug(__FILE__,"Entries in ICBArray: %d, IC-ID:%d", nic, id);
    TArtIC * ic = FindIC(id);

    if(NULL==ic){
      new ((*fICArray)[nic]) TArtIC();
      ic = (TArtIC *)fICArray->At(nic);
      ic->SetID(id);
      fICParaArray.push_back(para);
    }

    // set raw data
    if(ICB == detector){
      int ic_ch = para->FindCh(&mm);
      ic->SetRawADC(ic_ch,val);
    }
  }

  fDataLoaded=true;

  return;
}

