#include "TArtCalibSAMURAI.hh" 
#include "TArtCalibCoin.hh"
#include "TArtCalibDCHit.hh"
#include "TArtCalibDCTrack.hh"
#include "TArtCalibBPCHit.hh"
#include "TArtCalibBDC1Hit.hh"
#include "TArtCalibBDC2Hit.hh"
#include "TArtCalibFDC1Hit.hh"
#include "TArtCalibFDC2Hit.hh"
#include "TArtCalibBDC1Track.hh"
#include "TArtCalibBDC2Track.hh"
#include "TArtCalibFDC1Track.hh"
#include "TArtCalibFDC2Track.hh"
#include "TArtCalibHODPla.hh"
#include "TArtCalibSAMURAIPla.hh"
#include "TArtCalibSAMURAITZero.hh"
#include "TArtCalibICB.hh"
#include "TArtCalibICF.hh"
#include "TArtSAMURAIParameters.hh"
#include "TArtRawEventObject.hh"
#include "TArtCore.hh" 
#include "TArtStoreManager.hh"
#include "TArtReconstruction.hh"

#include <TROOT.h>
#include <TFile.h>
#include <TH2F.h>

#include "segidlist.hh"

//__________________________________________________________
TArtCalibSAMURAI::TArtCalibSAMURAI() : TArtReconstruction()
{
  TArtCore::Info(__FILE__,"Creating the SAMURAI detector objects...");
  sman = TArtStoreManager::Instance();

  // optimized with run 112
  //  Double_t bdc1_int_drift_tdc = 55;
  // run 140
  Double_t bdc1_int_drift_tdc = 795-740;
  Double_t bdc1_tzero = 795;
  //  Double_t bdc1_tzero = 805;
  //  Double_t bdc2_int_drift_tdc = 55;
  Double_t bdc2_int_drift_tdc = 770-722;
  //  Double_t bdc2_tzero = 800;
  Double_t bdc2_tzero = 770;
  Double_t fdc1_int_drift_tdc = 1610-1510;
  Double_t fdc1_tzero = 1610;
  Double_t fdc2_int_drift_tdc = 1495-1090;
  Double_t fdc2_tzero = 1495;

  fCalibCoin = new TArtCalibCoin();
  fCalibBPCHit = new TArtCalibBPCHit();
  fCalibBDC1Hit = new TArtCalibBDC1Hit();
  fCalibBDC2Hit = new TArtCalibBDC2Hit();
  fCalibFDC1Hit = new TArtCalibFDC1Hit();
  fCalibFDC2Hit = new TArtCalibFDC2Hit();
  fCalibBDC1Track = new TArtCalibBDC1Track(bdc1_int_drift_tdc,bdc1_tzero);
  fCalibBDC2Track = new TArtCalibBDC2Track(bdc2_int_drift_tdc,bdc2_tzero);
  fCalibFDC1Track = new TArtCalibFDC1Track(fdc1_int_drift_tdc,fdc1_tzero);
  fCalibFDC2Track = new TArtCalibFDC2Track(fdc2_int_drift_tdc,fdc2_tzero);
  fCalibHODPla = new TArtCalibHODPla();
  fCalibSAMURAIPla = new TArtCalibSAMURAIPla();
  fCalibSAMURAITZero = new TArtCalibSAMURAITZero();
  fCalibICB = new TArtCalibICB();
  fCalibICF = new TArtCalibICF();

}

//__________________________________________________________
TArtCalibSAMURAI::~TArtCalibSAMURAI()  {
  ClearData();
}

//__________________________________________________________
void TArtCalibSAMURAI::LoadDCTDCDistribution(char * file)  {

  TFile *fdcin = new TFile(file,"READ"); 
  gROOT->cd();
  TH1F *hist = NULL;

  hist = (TH1F*)((TH2F*)fdcin->Get("bdc1_ftdc_corr_0"))->ProjectionX();
  fCalibBDC1Track->SetTDCDistribution(hist);
  delete hist; hist = NULL;

  hist = (TH1F*)((TH2F*)fdcin->Get("bdc2_ftdc_corr_0"))->ProjectionX();
  fCalibBDC2Track->SetTDCDistribution(hist);
  delete hist; hist = NULL;

  hist = (TH1F*)((TH2F*)fdcin->Get("fdc1_ftdc_corr_0"))->ProjectionX();
  fCalibFDC1Track->SetTDCDistribution(hist);
  delete hist; hist = NULL;

  hist = (TH1F*)((TH2F*)fdcin->Get("fdc2_ftdc_corr_0"))->ProjectionX();
  fCalibFDC2Track->SetTDCDistribution(hist);
  delete hist; hist = NULL;

}

//__________________________________________________________
void TArtCalibSAMURAI::LoadData()   {

  TArtRawEventObject *fEvent = (TArtRawEventObject *)sman->FindDataContainer("RawEvent");
  for(int i=0;i<fEvent->GetNumSeg();i++){
    TArtRawSegmentObject *seg = fEvent->GetSegment(i);
    int device = seg->GetDevice();
    if(SAMURAI == device)
      LoadData(seg);
  }
  return;

}

//__________________________________________________________
void TArtCalibSAMURAI::LoadData(TArtRawSegmentObject *seg)   {

  int detector = seg->GetDetector();
  if(BDC == detector)
    fCalibBDC1Hit->LoadData(seg);
  if(BDC == detector)
    fCalibBDC2Hit->LoadData(seg);
  else if(FDC1 == detector)
    fCalibFDC1Hit->LoadData(seg);
  else if(FDC2 == detector)
    fCalibFDC2Hit->LoadData(seg);
  else if(BPC == detector)
    fCalibBPCHit->LoadData(seg);
  else if(BPC == detector)
    fCalibBPCHit->LoadData(seg);
  else if(COIN == detector)
    fCalibCoin->LoadData(seg);
  else if(HODFQ == detector || HODPQ == detector || HODFT == detector || HODPT == detector) fCalibHODPla->LoadData(seg);
  else if(PLAT == detector || PLAQ == detector) fCalibSAMURAIPla->LoadData(seg);
  else if(ICB == detector) fCalibICB->LoadData(seg);
  else if(ICF == detector) fCalibICF->LoadData(seg);

  fDataLoaded = true;
  return;

}

//__________________________________________________________
void TArtCalibSAMURAI::ClearData()   {

  fCalibBPCHit->ClearData();
  fCalibBDC1Hit->ClearData();
  fCalibBDC1Track->ClearData();
  fCalibBDC2Hit->ClearData();
  fCalibBDC2Track->ClearData();
  fCalibFDC1Hit->ClearData();
  fCalibFDC1Track->ClearData();
  fCalibFDC2Hit->ClearData();
  fCalibFDC2Track->ClearData();
  fCalibHODPla->ClearData();
  fCalibSAMURAIPla->ClearData();
  fCalibSAMURAITZero->ClearData();
  fCalibICB->ClearData();
  fCalibICF->ClearData();
  fDataLoaded = false;
  fReconstructed = false;
  return;

}

//__________________________________________________________
void TArtCalibSAMURAI::ReconstructData()   {

  if(!fDataLoaded) LoadData();
  fCalibBDC1Hit->ReconstructData();
  fCalibBDC1Track->ReconstructData();
  fCalibBDC2Hit->ReconstructData();
  fCalibBDC2Track->ReconstructData();
  fCalibBPCHit->ReconstructData();
  fCalibFDC1Hit->ReconstructData();
  fCalibFDC1Track->ReconstructData();
  fCalibFDC2Hit->ReconstructData();
  fCalibFDC2Track->ReconstructData();
  fCalibHODPla->ReconstructData();
  fCalibSAMURAIPla->ReconstructData();
  fCalibSAMURAITZero->ReconstructData();
  fCalibICB->ReconstructData();
  fCalibICF->ReconstructData();

  fReconstructed = true;
  return;
}

//__________________________________________________________
TClonesArray * TArtCalibSAMURAI::GetBPCHitArray(){
  return fCalibBPCHit->GetDCHitArray();
}
//__________________________________________________________
TClonesArray * TArtCalibSAMURAI::GetBDC1HitArray(){
  return fCalibBDC1Hit->GetDCHitArray();
}
//__________________________________________________________
TClonesArray * TArtCalibSAMURAI::GetBDC2HitArray(){
  return fCalibBDC2Hit->GetDCHitArray();
}
//__________________________________________________________
TClonesArray * TArtCalibSAMURAI::GetFDC1HitArray(){
  return fCalibFDC1Hit->GetDCHitArray();
}
//__________________________________________________________
TClonesArray * TArtCalibSAMURAI::GetFDC2HitArray(){
  return fCalibFDC2Hit->GetDCHitArray();
}
//__________________________________________________________
TClonesArray * TArtCalibSAMURAI::GetBDC1TrackArray(){
  return fCalibBDC1Track->GetDCTrackArray();
}
//__________________________________________________________
TClonesArray * TArtCalibSAMURAI::GetBDC2TrackArray(){
  return fCalibBDC2Track->GetDCTrackArray();
}
//__________________________________________________________
TClonesArray * TArtCalibSAMURAI::GetFDC1TrackArray(){
  return fCalibFDC1Track->GetDCTrackArray();
}
//__________________________________________________________
TClonesArray * TArtCalibSAMURAI::GetFDC2TrackArray(){
  return fCalibFDC2Track->GetDCTrackArray();
}
//__________________________________________________________
TClonesArray * TArtCalibSAMURAI::GetHODPlaArray(){
  return fCalibHODPla->GetHODPlaArray();
}
//__________________________________________________________
TClonesArray * TArtCalibSAMURAI::GetSAMURAIPlaArray(){
  return fCalibSAMURAIPla->GetPlasticArray();
}
//__________________________________________________________
TClonesArray * TArtCalibSAMURAI::GetSAMURAITZeroArray(){
  return fCalibSAMURAITZero->GetTZeroArray();
}
//__________________________________________________________
TClonesArray * TArtCalibSAMURAI::GetICBArray(){
  return fCalibICB->GetICArray();
}
//__________________________________________________________
TClonesArray * TArtCalibSAMURAI::GetICFArray(){
  return fCalibICF->GetICArray();
}
