#include "TArtCalibPlastic.hh" 
#include "TArtCalibSAMURAIPla.hh" 
#include "TArtPlastic.hh"
#include "TArtPlasticPara.hh"
#include "TArtSAMURAIParameters.hh"
#include "TArtRawEventObject.hh"
#include "TArtStoreManager.hh"
#include "TArtCore.hh" 
#include "segidlist.hh" 

#include <TROOT.h>

//__________________________________________________________
TArtCalibSAMURAIPla::TArtCalibSAMURAIPla() : TArtCalibPlastic("SAMURAIPla")
{
  TArtCore::Info(__FILE__,"Creating the SAMURAI-Plastic detector objects...");
}

//__________________________________________________________
TArtCalibSAMURAIPla::~TArtCalibSAMURAIPla()  {
}

//__________________________________________________________
void TArtCalibSAMURAIPla::LoadData(TArtRawSegmentObject *seg)   {

  TArtSAMURAIParameters *setup = (TArtSAMURAIParameters*)sman->FindParameters("SAMURAIParameters");
  if(!setup){
    TArtCore::Info(__FILE__,"Can not find parameter handler: SAMURAIParameters");
    throw;
  }

  int device   = seg->GetDevice();
  int fpl      = seg->GetFP();
  int detector = seg->GetDetector();
  if(device != SAMURAI) return;
  if(!(PLAT == detector || PLAQ == detector)) return;
  for(int j=0;j<seg->GetNumData();j++){
    TArtRawDataObject *d = seg->GetData(j);
    int geo = d->GetGeo(); 
    int ch = d->GetCh(); 
    unsigned int val = d->GetVal();
    TArtRIDFMap mm(fpl,detector,geo,ch);
    TArtPlasticPara *para = (TArtPlasticPara*)setup->FindPlasticPara(&mm);
    if(NULL == para){
      TArtCore::Error(__FILE__,"Could not find TArtPlasticPara...: Dev:%d, %s", device, mm.GetMapInfo());
      continue;
    }
    else {
      TArtCore::Debug(__FILE__,"Find TArtPlasticPara...: Dev:%d, %s, : %s", device, mm.GetMapInfo(), para->GetDetectorName()->Data());
    }

    int id = para->GetID();
    int npla = fPlasticArray->GetEntries();
    TArtCore::Debug(__FILE__,"Entries in PlasticArray: %d, Plastic-ID:%d", npla, id);
    TArtPlastic * pla = FindPlastic(id);

    if(NULL==pla){
      new ((*fPlasticArray)[npla]) TArtPlastic();
      pla = (TArtPlastic *)fPlasticArray->At(npla);
      pla->SetID(id);

      fPlasticParaArray.push_back(para);
    }

    // set raw data
    if(PLAQ == detector){
      if(mm==*((TArtRIDFMap *)para->GetQLMap()))
	pla->SetQLRaw(val);
      else if(mm==*((TArtRIDFMap *)para->GetQRMap()))
	pla->SetQRRaw(val);
    }
    if(PLAT == detector){
      if(mm==*((TArtRIDFMap *)para->GetTLMap()))
	pla->SetTLRaw(val);
      else if(mm==*((TArtRIDFMap *)para->GetTRMap()))
	pla->SetTRRaw(val);
    }

  }

  fDataLoaded=true;

  return;
}

